@@ -23,7 +23,7 @@ Attributes:
 All other functions and classes are private, and should not be used by other
 modules.
     
-$Id: simulator.py 646 2009-06-09 12:35:08Z apdavison $
+$Id: simulator.py 702 2010-02-03 10:39:32Z apdavison $
 """
 
 import logging
@@ -31,7 +31,7 @@ import brian
 import numpy
 from itertools import izip
 import scipy.sparse
-from pyNN import common, cells, recording
+from pyNN import common, cells
 
 mV = brian.mV
 ms = brian.ms
@@ -43,6 +43,7 @@ Hz = brian.Hz
 recorder_list = []
 ZERO_WEIGHT = 1e-99
 
+logger = logging.getLogger("PyNN")
 
 # --- Internal Brian functionality -------------------------------------------- 
 
@@ -179,6 +180,9 @@ class _State(object):
 def reset():
     """Reset the state of the current network to time t = 0."""
     state.simclock.reinit()
+    for device in net.operations:
+        if hasattr(device, "reinit"):
+            device.reinit()
 
 def run(simtime):
     """Advance the simulation for a certain time."""
@@ -223,7 +227,7 @@ class ID(int, common.IDMixin):
         for name, value in parameters.items():
             if name in ['v_thresh', 'v_reset', 'tau_refrac', 'start', 'rate', 'duration']:
                 setattr(self.parent_group, name, value)
-                logging.warning("This parameter cannot be set for individual cells within a Population. Changing the value for all cells in the Population.")
+                logger.warning("This parameter cannot be set for individual cells within a Population. Changing the value for all cells in the Population.")
             elif name == 'spiketimes':
                 #setattr(self.parent_group, name, [value]*len(self.parent_group))
                 self.parent_group.spiketimes[int(self)] = value
@@ -231,115 +235,6 @@ class ID(int, common.IDMixin):
                 #    raise IndexError("%s. index=%d, self.parent_group.spiketimes=%s" % (errmsg, int(self), self.parent_group.spiketimes))
             else:
                 setattr(self.parent_group[int(self)], name, value)
-
-
-# --- For implementation of record_X()/get_X()/print_X() -----------------------
-
-class Recorder(object):
-    """Encapsulates data and functions related to recording model variables."""
-  
-    numpy1_1_formats = {'spikes': "%d\t%g",
-                        'v': "%d\t%g\t%g"}
-    numpy1_0_formats = {'spikes': "%g", # only later versions of numpy support different
-                        'v': "%g",
-                        'gsyn': "%g",}      # formats for different columns
-    formats = {'spikes': 'id t',
-               'v': 'id t v',
-               'gsyn':'id t ge gi'} #???
-  
-    def __init__(self, variable, population=None, file=None):
-        """
-        Create a recorder.
-        
-        `variable` -- "spikes", "v" or "gsyn"
-        `population` -- the Population instance which is being recorded by the
-                        recorder (optional)
-        `file` -- one of:
-            - a file-name,
-            - `None` (write to a temporary file)
-            - `False` (write to memory).
-        """
-        assert variable in Recorder.formats
-        self.variable = variable
-        self.filename = file or None
-        self.population = population # needed for writing header information
-        self.recorded = set([])
-        self._devices = [] # defer creation until first call of record()
-    
-    def _create_devices(self, group):
-        """Create a Brian recording device."""
-        if self.variable == 'spikes':
-            devices = [brian.SpikeMonitor(group, record=True)]
-        elif self.variable == 'v':
-            devices = [brian.StateMonitor(group, 'v', record=True, clock=state.simclock)]
-        elif self.variable == 'gsyn':
-            example_cell = list(self.recorded)[0]
-            varname = example_cell.cellclass.synapses['excitatory']
-            device1 = brian.StateMonitor(group, varname, record=True, clock=state.simclock)
-            varname = example_cell.cellclass.synapses['inhibitory']
-            device2 = brian.StateMonitor(group, varname, record=True, clock=state.simclock)
-            devices = [device1, device2]
-        for device in devices:
-            net.add(device)
-        return devices
-    
-    def record(self, ids):
-        """Add the cells in `ids` to the set of recorded cells."""
-        #update StateMonitor.record and StateMonitor.recordindex
-        self.recorded = self.recorded.union(ids)
-        if len(self._devices) == 0:
-            self._devices = self._create_devices(ids[0].parent_group)
-        if self.variable is not 'spikes':
-            for device in self._devices:
-                device.record = list(self.recorded)
-                device.recordindex = dict((i,j) for i,j in zip(device.record,
-                                                               range(len(device.record))))
-    
-    def get(self, gather=False, compatible_output=True):
-        """Return the recorded data as a Numpy array."""
-        if self.population:
-            offset = self.population.first_id
-        else:
-            offset = 0
-        if self.variable == 'spikes':
-            data = numpy.array([(id, time/ms) for (id, time) in self._devices[0].spikes if id in self.recorded])
-        elif self.variable == 'v':
-            values = self._devices[0].values/mV
-            times = self._devices[0].times/ms
-            data = numpy.empty((0,3))
-            for id, row in enumerate(values):
-                new_data = numpy.array([numpy.ones(row.shape)*(id-offset), times, row]).T
-                data = numpy.concatenate((data, new_data))
-        elif self.variable == 'gsyn':
-            values1 = self._devices[0].values/uS
-            values2 = self._devices[1].values/uS
-            times = self._devices[0].times/ms
-            data = numpy.empty((0,4))
-            for id, (row1, row2) in enumerate(zip(values1, values2)):
-                assert row1.shape == row2.shape
-                new_data = numpy.array([numpy.ones(row1.shape)*(id-offset), times, row1, row2]).T
-                data = numpy.concatenate((data, new_data))
-        return data
-        
-    def write(self, file=None, gather=False, compatible_output=True):
-        """Write recorded data to file."""
-        data = self.get(gather)
-        filename = file or self.filename
-        recording.rename_existing(filename)
-        try:
-            numpy.savetxt(filename, data, Recorder.numpy1_0_formats[self.variable], delimiter='\t')
-        except AttributeError, errmsg:
-            # we assume the error is due to the lack of savetxt in older versions of numpy and
-            # so provide a cut-down version of that function
-            f = open(filename, 'w')
-            fmt = Recorder.numpy1_0_formats[self.variable]
-            for row in data:
-                f.write('\t'.join([fmt%val for val in row]) + '\n')
-            f.close()
-        if compatible_output:
-            recording.write_compatible_output(filename, filename, self.variable,
-                                              Recorder.formats[self.variable],
-                                              self.population, state.dt)
         
     
 # --- For implementation of create() and Population.__init__() ----------------- 
@@ -463,6 +358,7 @@ class Connection(object):
 
     def _get_weight(self):
         """Synaptic weight in nA or µS."""
+        ###print "in Connection._get_weight(), weight_units = %s" % self.bc.weight_units
         return float(self.bc[self.addr]/self.bc.weight_units)
 
     def _set_delay(self, d):
@@ -482,13 +378,17 @@ class ConnectionManager(object):
     accessing individual connections.
     """
 
-    def __init__(self, synapse_model=None, parent=None):
+    def __init__(self, synapse_type, synapse_model=None, parent=None):
         """
         Create a new ConnectionManager.
         
+        `synapse_type` -- the 'physiological type' of the synapse, e.g.
+                          'excitatory' or 'inhibitory',or any other key in the
+                          `synapses` attibute of the celltype class.
         `synapse_model` -- not used. Present for consistency with other simulators.
         `parent` -- the parent `Projection`, if any.
         """
+        self.synapse_type = synapse_type
         self.parent = parent
         self.connections = {}
         self.n = 0
@@ -556,7 +456,7 @@ class ConnectionManager(object):
             self.connections[syn_id][src_id][tgt_id] = bc
         return bc
     
-    def connect(self, source, targets, weights, delays, synapse_type):
+    def connect(self, source, targets, weights, delays):
         """
         Connect a neuron to one or more other neurons with a static connection.
         
@@ -566,10 +466,6 @@ class ConnectionManager(object):
                      Must have the same length as `targets`.
         `delays`  -- a list/1D array of connection delays, or a single delay.
                      Must have the same length as `targets`.
-        `synapse_type` -- a string identifying the synapse to connect to. Should
-                          be "excitatory", "inhibitory", or any other key in the
-                          `synapses` attibute of the celltype class. May be
-                          `None`, which is treated the same as "excitatory".
         """
         #print "connecting", source, "to", targets, "with weights", weights, "and delays", delays
         if not common.is_listlike(targets):
@@ -590,7 +486,7 @@ class ConnectionManager(object):
         else:
             units = nA
             
-        synapse_type = synapse_type or "excitatory"
+        synapse_type = self.synapse_type or "excitatory"
         synapse_obj  = targets[0].cellclass.synapses[synapse_type]
         try:
             source_group = source.parent_group
@@ -633,7 +529,8 @@ class ConnectionManager(object):
         if self.parent is None:
             raise Exception("Only implemented for connections created via a Projection object, not using connect()")
         synapse_obj = self.parent.post.celltype.synapses[self.parent.target or "excitatory"]
-        weight_units = uS and ("cond" in self.parent.post.celltype.__class__.__name__) or nA
+        weight_units = ("cond" in self.parent.post.celltype.__class__.__name__) and uS or nA
+        ###print "in ConnectionManager.get(), weight_units = %s" % weight_units
         bc = self._get_brian_connection(self.parent.pre.brian_cells,
                                         self.parent.post.brian_cells,
                                         synapse_obj,
@@ -666,12 +563,15 @@ class ConnectionManager(object):
         
         `name`  -- attribute name
         `value` -- the attribute numeric value, or a list/1D array of such
-                   values of the same length as the number of local connections.
+                   values of the same length as the number of local connections,
+                   or a 2D array with the same dimensions as the connectivity
+                   matrix (as returned by `get(format='array')`).
         """
         if self.parent is None:
             raise Exception("Only implemented for connections created via a Projection object, not using connect()")
         synapse_obj = self.parent.post.celltype.synapses[self.parent.target or "excitatory"]
-        weight_units = uS and ("cond" in self.parent.post.celltype.__class__.__name__) or nA
+        weight_units = ("cond" in self.parent.post.celltype.__class__.__name__) and uS or nA
+        ###print "in ConnectionManager.set(), weight_units = %s" % weight_units
         bc = self._get_brian_connection(self.parent.pre.brian_cells,
                                         self.parent.post.brian_cells,
                                         synapse_obj,
@@ -688,6 +588,10 @@ class ConnectionManager(object):
             for row in M.data:
                 for i in range(len(row)):
                     row[i] = value*units
+        elif isinstance(value, numpy.ndarray) and len(value.shape) == 2:
+            address_gen = ((i,j) for i,row in enumerate(bc.W.rows) for j in row)
+            for (i,j) in address_gen:
+                M[i,j] = value[i,j]*units
         elif common.is_listlike(value):
             assert len(value) == M.getnnz()
             address_gen = ((i,j) for i,row in enumerate(bc.W.rows) for j in row)
