@@ -69,14 +69,15 @@ Common API implementation/base classes:
     STDPWeightDependence
     STDPTimingDependence
 
-$Id: common.py 650 2009-06-09 12:50:03Z apdavison $
+$Id: common.py 710 2010-02-12 15:31:52Z apdavison $
 """
 
 import types, copy, sys
 import numpy
 import logging
 from math import *
-from pyNN import random, utility
+import operator
+from pyNN import random, utility, recording
 from string import Template
 
 if not 'simulator' in locals():
@@ -85,9 +86,11 @@ if not 'simulator' in locals():
 DEFAULT_WEIGHT = 0.0
 DEFAULT_BUFFER_SIZE = 10000
 
-class InvalidParameterValueError(Exception):
+logger = logging.getLogger("PyNN")
+
+class InvalidParameterValueError(ValueError):
     """Inappropriate parameter value"""
-    pass # should subclass ValueError?
+    pass
 
 class NonExistentParameterError(Exception):
     """
@@ -170,22 +173,25 @@ def is_conductance(target_cell):
     Returns True if the target cell uses conductance-based synapses, False if it
     uses current-based synapses, and None if the synapse-basis cannot be determined.
     """
-    if target_cell.local and hasattr(target_cell, 'cellclass'):
+    if hasattr(target_cell, 'local') and target_cell.local and hasattr(target_cell, 'cellclass'):
         if isinstance(target_cell.cellclass, type):
-            is_conductance = "cond" in target_cell.cellclass.__name__
-        else:
+            is_conductance = target_cell.cellclass.conductance_based
+        else: # where cellclass is a string, i.e. for native cell types in NEST
             is_conductance = "cond" in target_cell.cellclass
     else:
         is_conductance = None
     return is_conductance
 
 def check_weight(weight, synapse_type, is_conductance):
+    #print "check_weight: node=%s, weight=%s, synapse_type=%s, is_conductance=%s" % (rank(), weight, synapse_type, is_conductance)
     if weight is None:
         weight = DEFAULT_WEIGHT
     if is_listlike(weight):
         weight = numpy.array(weight)
-        all_negative = (weight<=0).all()
-        all_positive = (weight>=0).all()
+        nan_filter = (1-numpy.isnan(weight)).astype(bool) # weight arrays may contain NaN, which should be ignored
+        filtered_weight = weight[nan_filter]
+        all_negative = (filtered_weight<=0).all()
+        all_positive = (filtered_weight>=0).all()
         if not (all_negative or all_positive):
             raise InvalidWeightError("Weights must be either all positive or all negative")
     elif is_number(weight):
@@ -196,11 +202,11 @@ def check_weight(weight, synapse_type, is_conductance):
     if is_conductance or synapse_type == 'excitatory':
         if not all_positive:
             raise InvalidWeightError("Weights must be positive for conductance-based and/or excitatory synapses")
-    elif synapse_type == 'inhibitory':
+    elif is_conductance==False and synapse_type == 'inhibitory':
         if not all_negative:
             raise InvalidWeightError("Weights must be negative for current-based, inhibitory synapses")
     else: # is_conductance is None. This happens if the cell does not exist on the current node.
-        logging.debug("Can't check weight, conductance status unknown.")
+        logger.debug("Can't check weight, conductance status unknown.")
     return weight
 
 def check_delay(delay):
@@ -230,63 +236,58 @@ def distance(src, tgt, mask=None, scale_factor=1.0, offset=0.0,
         d = d[mask]
     return numpy.sqrt(numpy.dot(d, d))
 
-def distances(pre, post, mask=None, scale_factor=1.0, offset=0.0,
-              periodic_boundaries=None):
-    """
-    Calculate the entire distance matrix at once.
-    From http://projects.scipy.org/pipermail/numpy-discussion/2007-April/027203.html
-    """
-    # Note that `mask` is not used.
-    if isinstance(pre, Population):
-        x = pre.positions
-    else: 
-        x = pre.position
-        x = x.reshape(3, 1)
-    if isinstance(post, Population):
-        y = post.positions
-    else: 
-        y = post.position
-        y = y.reshape(3, 1)
-    y = scale_factor*(y + offset)
-    d = numpy.zeros((x.shape[1], y.shape[1]), dtype=x.dtype)
-    for i in xrange(x.shape[0]):
-        diff2 = abs(x[i,:,None] - y[i,:])
-        if periodic_boundaries is not None:
-            dims  = diff2.shape
-            diff2 = diff2.flatten()
-            diff2 = numpy.minimum(diff2, periodic_boundaries[i]-diff2)
-            diff2 = diff2.reshape(dims)
-        diff2 **= 2
-        d += diff2
-    numpy.sqrt(d, d)
-    return d
 
-class ConstIter(object):
-    """An iterator that always returns the same value."""
-    def __init__(self, x):
-        self.x = x
-    def next(self):
-        return self.x
+class Space(object):
+    
+    AXES = {'x' : [0],    'y': [1],    'z': [2],
+            'xy': [0,1], 'yz': [1,2], 'xz': [0,2], 'xyz': range(3), None: range(3)}
+    
+    def __init__(self, axes=None, scale_factor=1.0, offset=0.0,
+                 periodic_boundaries=None):
+        """
+        axes -- if not supplied, then the 3D distance is calculated. If supplied,
+            axes should be a string containing the axes to be used, e.g. 'x', or
+            'yz'. axes='xyz' is the same as axes=None.
+        scale_factor -- it may be that the pre and post populations use
+            different units for position, e.g. degrees and µm. In this case,
+            `scale_factor` can be specified, which is applied to the positions
+            in the post-synaptic population.
+        offset -- if the origins of the coordinate systems of the pre- and post-
+            synaptic populations are different, `offset` can be used to adjust
+            for this difference. The offset is applied before any scaling.
+        periodic_boundaries -- either `None`, or a tuple giving the boundaries
+            for each dimension, e.g. `((x_min, x_max), None, (z_min, z_max))`.
+        """
+        self.periodic_boundaries = periodic_boundaries
+        self.axes = numpy.array(Space.AXES[axes])
+        self.scale_factor = scale_factor
+        self.offset = offset
+        
+    def distances(self, A, B):
+        """
+        Calculate the distance matrix between two sets of coordinates, given
+        the topology of the current space.
+        From http://projects.scipy.org/pipermail/numpy-discussion/2007-April/027203.html
+        """
+        if len(A.shape) == 1:
+            A = A.reshape(3, 1)
+        if len(B.shape) == 1:
+            B = B.reshape(3, 1)
+        B = self.scale_factor*(B + self.offset)
+        d = numpy.zeros((A.shape[1], B.shape[1]), dtype=A.dtype)
+        for axis in self.axes:
+            diff2 = A[axis,:,None] - B[axis,:]
+            if self.periodic_boundaries is not None:
+                boundaries = self.periodic_boundaries[axis]
+                if boundaries is not None:
+                    range = boundaries[1]-boundaries[0]
+                    ad2 = abs(diff2)
+                    diff2 = numpy.minimum(ad2, range-ad2)
+            diff2 **= 2
+            d += diff2
+        numpy.sqrt(d, d)
+        return d
 
-def next_n(sequence, N, start_index, mask_local):
-    assert isinstance(N, int), "N is %s, should be an integer" % N
-    if isinstance(sequence, random.RandomDistribution):
-        values = numpy.array(sequence.next(N, mask_local=mask_local))
-    elif isinstance(sequence, (int, float)):
-        if mask_local is not None:
-            assert mask_local.size == N
-            N = mask_local.sum()
-            assert isinstance(N, int), "N is %s, should be an integer" % N
-        values = numpy.ones((N,))*float(sequence)
-    elif hasattr(sequence, "__len__"):
-        values = numpy.array(sequence[start_index:start_index+N], float)
-        if mask_local is not None:
-            assert mask_local.size == N
-            assert len(mask_local.shape) == 1, mask_local.shape
-            values = values[mask_local]
-    else:
-        raise Exception("sequence is of type %s" % type(sequence))
-    return values
 
 # ==============================================================================
 #   Accessing individual neurons
@@ -315,7 +316,7 @@ class IDMixin(object):
         self.local = True
 
     def __getattr__(self, name):
-        if name in self.non_parameter_attributes:
+        if name in self.non_parameter_attributes or (self.local and not self.is_standard_cell()):
             val = self.__getattribute__(name)
         else:
             try:
@@ -325,7 +326,7 @@ class IDMixin(object):
         return val
     
     def __setattr__(self, name, value):
-        if name in self.non_parameter_attributes:
+        if name in self.non_parameter_attributes or (self.local and not self.is_standard_cell()):
             object.__setattr__(self, name, value)
         else:
             return self.set_parameters(**{name:value})
@@ -571,6 +572,8 @@ class StandardCellType(StandardModelType):
 
     recordable = ['spikes', 'v', 'gsyn']
     synapse_types = ('excitatory', 'inhibitory')
+    conductance_based = True # over-ride for cells with current-based synapses
+    always_local = False # over-ride for NEST spike sources
 
 
 class ModelNotAvailable(object):
@@ -583,7 +586,7 @@ class ModelNotAvailable(object):
 #   Functions for simulation set-up and control
 # ==============================================================================
 
-def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False,
+def setup(timestep=0.1, min_delay=0.1, max_delay=10.0,
           **extra_params):
     """
     Should be called at the very beginning of a script.
@@ -599,20 +602,6 @@ def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False,
     if min_delay < timestep:
         "min_delay (%g) must be greater than timestep (%g)" % (min_delay, timestep)
     
-    if simulator:
-        backend = simulator.__name__.replace('.simulator', '')
-        log_file = "%s.log" % backend
-    else:
-        log_file = "pyNN.log"
-    if debug:
-        if isinstance(debug, basestring):
-            log_file = debug
-    if simulator and not simulator.state.initialized:
-        utility.init_logging(log_file, debug, num_processes(), rank())
-        logging.info("Initialization of %s (use setup(.., debug=True) to see a full logfile)" % backend)
-        simulator.state.initialized = True
-    
-    
 def end(compatible_output=True):
     """Do any necessary cleaning up before exiting."""
     raise NotImplementedError
@@ -621,6 +610,13 @@ def run(simtime):
     """Run the simulation for simtime ms."""
     raise NotImplementedError
 
+def reset():
+    """
+    Reset the time to zero, neuron membrane potentials and synaptic weights to
+    their initial values, and delete any recorded data. The network structure is
+    not changed, nor is the specification of which neurons to record from."""
+    simulator.reset()
+
 def get_current_time():
     """Return the current time in the simulation."""
     return simulator.state.t
@@ -675,7 +671,7 @@ def connect(source, target, weight=None, delay=None, synapse_type=None, p=1, rng
     """
     # This duplicates code from the Connector/FixedProbabilityConnector classes
     # should refactor to eliminate this repetition
-    logging.debug("connecting %s to %s on host %d" % (source, target, rank()))
+    logger.debug("connecting %s to %s on host %d" % (source, target, rank()))
     if not is_listlike(source):
         source = [source]
     if not is_listlike(target):
@@ -683,17 +679,17 @@ def connect(source, target, weight=None, delay=None, synapse_type=None, p=1, rng
     delay = check_delay(delay)
     if p < 1:
         rng = rng or numpy.random
-    connection_manager = simulator.ConnectionManager()
+    connection_manager = simulator.ConnectionManager(synapse_type)
     for tgt in target:
-        if not isinstance(tgt, IDMixin):
-            raise ConnectionError("target is not a cell, actually %s" % type(tgt))
+        #if not isinstance(tgt, IDMixin):
+        #    raise ConnectionError("target is not a cell, actually %s" % type(tgt))
         sources = numpy.array(source, dtype=type(source))
         if p < 1:
             rarr = rng.uniform(0, 1, len(source))
             sources = sources[rarr<p]
         weight = check_weight(weight, synapse_type, is_conductance(tgt))
         for src in sources:
-            connection_manager.connect(src, tgt, weight, delay, synapse_type)
+            connection_manager.connect(src, tgt, weight, delay)
     return connection_manager
 
 
@@ -746,6 +742,7 @@ class Population(object):
     An array of neurons all of the same type. `Population' is used as a generic
     term intended to include layers, columns, nuclei, etc., of cells.
     """
+    nPop = 0
     
     def __init__(self, dims, cellclass, cellparams=None, label=None, parent=None):
         """
@@ -767,14 +764,18 @@ class Population(object):
             self.dim = (self.dim,)
         else:
             assert isinstance(dims, tuple), "`dims` must be an integer or a tuple. You have supplied a %s" % type(dims)
-        self.label = label
-        self.celltype = cellclass
+        self.label = label or 'population%d' % Population.nPop         
+        if isinstance(cellclass, type) and issubclass(cellclass, StandardCellType):
+            self.celltype = cellclass(cellparams)
+        else:
+            self.celltype = cellclass
         self.ndim = len(self.dim)
         self.cellparams = cellparams
-        self.size = self.dim[0]
         self.parent = parent
+        self.size = self.dim[0]
         for i in range(1, self.ndim):
             self.size *= self.dim[i]
+        Population.nPop += 1
     
     def __getitem__(self, addr):
         """
@@ -881,7 +882,7 @@ class Population(object):
 
     def _set_positions(self, pos_array):
         assert isinstance(pos_array, numpy.ndarray)
-        assert pos_array.shape == (3, self.size)
+        assert pos_array.shape == (3, self.size), "%s != %s" % (pos_array.shape, (3, self.size))
         self._positions = pos_array.copy() # take a copy in case pos_array is changed later
 
     positions = property(_get_positions, _set_positions,
@@ -936,7 +937,7 @@ class Population(object):
             param_dict = param
         else:
             raise InvalidParameterValueError
-        logging.info("%s.set(%s)", self.label, param_dict)
+        logger.debug("%s.set(%s)", self.label, param_dict)
         for cell in self:
             cell.set_parameters(**param_dict)
 
@@ -956,11 +957,11 @@ class Population(object):
         assert local_values.shape[0] == self.local_cells.size, "%d != %d" % (local_values.size, self.local_cells.size)
         
         try:
-            logging.info("%s.tset('%s', array(shape=%s, min=%s, max=%s))",
+            logger.debug("%s.tset('%s', array(shape=%s, min=%s, max=%s))",
                          self.label, parametername, value_array.shape,
                          value_array.min(), value_array.max())
         except TypeError: # min() and max() won't work for non-numeric values
-            logging.info("%s.tset('%s', non_numeric_array(shape=%s))",
+            logger.debug("%s.tset('%s', non_numeric_array(shape=%s))",
                          self.label, parametername, value_array.shape)
         
         # Set the values for each cell
@@ -974,7 +975,7 @@ class Population(object):
         """
         rarr = rand_distr.next(n=self.all_cells.size, mask_local=self._mask_local.flatten())
         rarr = numpy.array(rarr)
-        logging.info("%s.rset('%s', %s)", self.label, parametername, rand_distr)
+        logger.debug("%s.rset('%s', %s)", self.label, parametername, rand_distr)
         for cell,val in zip(self, rarr):
             setattr(cell, parametername, val)
     
@@ -1005,7 +1006,10 @@ class Population(object):
 
     def can_record(self, variable):
         """Determine whether `variable` can be recorded from this population."""
-        return (variable in self.celltype.recordable)
+        if isinstance(self.celltype, StandardCellType):
+            return (variable in self.celltype.recordable)
+        else:
+            return True # for now, not able to check for native cells, although it should be possible in principle
 
     def _record(self, variable, record_from=None, rng=None, to_file=True):
         """
@@ -1018,18 +1022,16 @@ class Population(object):
         elif record_from is None: # record from all cells:
             record_from = self.all_cells.flatten()
         elif isinstance(record_from, int): # record from a number of cells, selected at random  
-            # Each node will record N/nhost cells...
-            nrec = int(record_from/num_processes())
+            nrec = record_from
             if not rng:
                 rng = random.NumpyRNG()
             record_from = rng.permutation(self.all_cells.flatten())[0:nrec]
-            # need ID objects, permutation returns integers
-            # ???
+            logger.debug("The %d cells recorded have IDs %s" % (nrec, record_from))
         else:
             raise Exception("record_from must be either a list of cells or the number of cells to record from")
         # record_from is now a list or numpy array. We do not have to worry about whether the cells are
         # local because the Recorder object takes care of this.
-        logging.info("%s.record('%s', %s)", self.label, variable, record_from[:5])
+        logger.debug("%s.record('%s', %s)", self.label, variable, record_from[:5])
         self.recorders[variable].record(record_from)
 
     def record(self, record_from=None, rng=None, to_file=True):
@@ -1061,10 +1063,12 @@ class Population(object):
         """
         self._record('gsyn', record_from, rng, to_file)
 
-    def printSpikes(self, filename, gather=True, compatible_output=True):
+    def printSpikes(self, file, gather=True, compatible_output=True):
         """
         Write spike times to file.
         
+        file should be either a filename or a PyNN File object.
+        
         If compatible_output is True, the format is "spiketime cell_id",
         where cell_id is the index of the cell counting along rows and down
         columns (and the extension of that for 3-D).
@@ -1082,7 +1086,7 @@ class Population(object):
         file will be written on each node, containing only the cells simulated
         on that node.
         """        
-        self.recorders['spikes'].write(filename, gather, compatible_output)
+        self.recorders['spikes'].write(file, gather, compatible_output)
     
     def getSpikes(self, gather=True, compatible_output=True):
         """
@@ -1093,10 +1097,12 @@ class Population(object):
         """
         return self.recorders['spikes'].get(gather, compatible_output)
 
-    def print_v(self, filename, gather=True, compatible_output=True):
+    def print_v(self, file, gather=True, compatible_output=True):
         """
         Write membrane potential traces to file.
         
+        file should be either a filename or a PyNN File object.
+        
         If compatible_output is True, the format is "v cell_id",
         where cell_id is the index of the cell counting along rows and down
         columns (and the extension of that for 3-D).
@@ -1112,7 +1118,7 @@ class Population(object):
         file will be written on each node, containing only the cells simulated
         on that node.
         """
-        self.recorders['v'].write(filename, gather, compatible_output)
+        self.recorders['v'].write(file, gather, compatible_output)
     
     def get_v(self, gather=True, compatible_output=True):
         """
@@ -1121,10 +1127,12 @@ class Population(object):
         """
         return self.recorders['v'].get(gather, compatible_output)
     
-    def print_gsyn(self, filename, gather=True, compatible_output=True):
+    def print_gsyn(self, file, gather=True, compatible_output=True):
         """
         Write synaptic conductance traces to file.
         
+        file should be either a filename or a PyNN File object.
+        
         If compatible_output is True, the format is "t g cell_id",
         where cell_id is the index of the cell counting along rows and down
         columns (and the extension of that for 3-D).
@@ -1135,7 +1143,7 @@ class Population(object):
         is used. This may be faster, since it avoids any post-processing of the
         voltage files.
         """
-        self.recorders['gsyn'].write(filename, gather, compatible_output)
+        self.recorders['gsyn'].write(file, gather, compatible_output)
     
     def get_gsyn(self, gather=True, compatible_output=True):
         """
@@ -1144,15 +1152,22 @@ class Population(object):
         """
         return self.recorders['gsyn'].get(gather, compatible_output)
         
+    def get_spike_counts(self, gather=True):
+        """
+        Returns the number of spikes for each neuron.
+        """
+        return self.recorders['spikes'].count(gather)
+        
     def meanSpikeCount(self, gather=True):
         """
         Returns the mean number of spikes per neuron.
         """
         spike_counts = self.recorders['spikes'].count(gather)
         total_spikes = sum(spike_counts.values())
-        return float(total_spikes)/len(spike_counts)
+        if rank() == 0 or not gather: # should maybe use allgather, and get the numbers on all nodes
+            return float(total_spikes)/len(spike_counts)
     
-    def describe(self, template='standard'):
+    def describe(self, template='standard', fill=lambda t,c: Template(t).safe_substitute(c)):
         """
         Returns a human-readable description of the population
         """
@@ -1185,7 +1200,10 @@ class Population(object):
         if len(self.local_cells) > 0:
             first_id = self.local_cells[0]
             context.update(local_first_id=first_id)
-            context.update(cell_parameters=first_id.get_parameters())
+            cell_parameters = first_id.get_parameters()
+            names = cell_parameters.keys()
+            names.sort()
+            context.update(cell_parameters=[(name,cell_parameters[name]) for name in names])
         context.update(celltype=self.celltype.__class__.__name__)
         context.update(n_cells=len(self))
         context.update(n_cells_local=len(self.local_cells))
@@ -1196,7 +1214,7 @@ class Population(object):
         if template == None:
             return context
         else:
-            return Template(template).substitute(context)
+            return fill(template, context)
     
     def inject(self, current_source):
         """
@@ -1224,18 +1242,21 @@ class Projection(object):
         presynaptic_population and postsynaptic_population - Population objects.
         
         source - string specifying which attribute of the presynaptic cell
-                 signals action potentials
+                 signals action potentials. This is only needed for
+                 multicompartmental cells with branching axons or dendrodendritic
+                 synapses. All standard cells have a single source, and this
+                 is the default.
                  
         target - string specifying which synapse on the postsynaptic cell to
-                 connect to
-                 
-        If source and/or target are not given, default values are used.
+                 connect to. For standard cells, this can be 'excitatory' or
+                 'inhibitory'. For non-standard cells, it could be 'NMDA', etc.
+                 If target is not given, the default values of 'excitatory' is used.
         
         method - a Connector object, encapsulating the algorithm to use for
                  connecting the neurons.
         
         synapse_dynamics - a `SynapseDynamics` object specifying which
-        synaptic plasticity mechanisms to use.
+                 synaptic plasticity mechanisms to use.
         
         rng - specify an RNG object to be used by the Connector.
         """
@@ -1286,28 +1307,25 @@ class Projection(object):
                 elif len(possible_models) == 0 :
                     raise NoModelAvailableError("No available plasticity models")
                 elif len(possible_models) > 1 :
-                    if self.synapse_dynamics.slow.model:
-                        # addition of the `model` attribute (see r415) is a pragmatic solution
-                        # but not an elegant one, and I don't think it should go into a released API
-                        # Since the problem of multiple models only seems to appear for NEST
-                        # with homogeneous and non-homogeneous versions, it would be better either
-                        # for the code to decide itself which to use (would be complex, as
-                        # connection creation would have to be deferred to run()) or to have
-                        # a global OPTIMIZED variable (possibly set in setup()) - if this was
-                        # set True the homogeneous version would be used and later attempts to
-                        # change parameters of the synapse would raise Exceptions.
-                        if self.synapse_dynamics.slow.model in list(possible_models):
-                            self.long_term_plasticity_mechanism = self.synapse_dynamics.slow.model
-                        else:
-                            print "The particular model %s does not exist !" %self.synapse_dynamics.slow.model
-                    else:
-                        # we pass the set of models back to the simulator-specific module for it to deal with
-                        self.long_term_plasticity_mechanism = possible_models
+                    # we pass the set of models back to the simulator-specific module for it to deal with
+                    self.long_term_plasticity_mechanism = possible_models
                      
     def __len__(self):
-        """Return the total number of connections."""
+        """Return the total number of local connections."""
         return len(self.connection_manager)
     
+    def size(self, gather=True):
+        """
+        Return the total number of connections.
+            - only local connections, if gather is False,
+            - all connections, if gather is True (default)
+        """
+        if gather:
+            n = len(self)
+            return recording.mpi_sum(n)
+        else:
+            return len(self)
+    
     def __repr__(self):
         return 'Projection("%s")' % self.label
     
@@ -1317,11 +1335,13 @@ class Projection(object):
         """
         w can be a single number, in which case all weights are set to this
         value, or a list/1D array of length equal to the number of connections
-        in the projection.
+        in the projection, or a 2D array with the same dimensions as the
+        connectivity matrix (as returned by `getWeights(format='array')`).
         Weights should be in nA for current-based and µS for conductance-based
         synapses.
         """
         # should perhaps add a "distribute" argument, for symmetry with "gather" in getWeights()
+        w = check_weight(w, self.synapse_type, is_conductance(self.post.local_cells[0]))
         self.connection_manager.set('weight', w)
     
     def randomizeWeights(self, rand_distr):
@@ -1337,7 +1357,8 @@ class Projection(object):
         """
         d can be a single number, in which case all delays are set to this
         value, or a list/1D array of length equal to the number of connections
-        in the projection.
+        in the projection, or a 2D array with the same dimensions as the
+        connectivity matrix (as returned by `getDelays(format='array')`).
         """
         self.connection_manager.set('delay', d)
     
@@ -1346,6 +1367,19 @@ class Projection(object):
         Set delays to random values taken from rand_distr.
         """
         self.setDelays(rand_distr.next(len(self)))
+
+    def setSynapseDynamics(self, param, value):
+        """
+        Set parameters of the dynamic synapses for all connections in this
+        projection.
+        """
+        self.connection_manager.set(param, value)
+
+    def randomizeSynapseDynamics(self, param, rand_distr):
+        """
+        Set parameters of the synapse dynamics to values taken from rand_distr
+        """
+        self.setSynapseDynamics(param, rand_distr.next(len(self)))
     
     # --- Methods for writing/reading information to/from file. ----------------
     
@@ -1359,7 +1393,7 @@ class Projection(object):
         one connection between two cells, the summed weight will be given.
         """
         if gather:
-            logging.error("getWeights() with gather=True not yet implemented")
+            logger.error("getWeights() with gather=True not yet implemented")
         return self.connection_manager.get('weight', format, offset=(self.pre.first_id, self.post.first_id))
             
     def getDelays(self, format='list', gather=True):
@@ -1371,30 +1405,56 @@ class Projection(object):
         connections).
         """
         if gather:
-            logging.error("getDelays() with gather=True not yet implemented")
+            logger.error("getDelays() with gather=True not yet implemented")
         return self.connection_manager.get('delay', format, offset=(self.pre.first_id, self.post.first_id))
+
+    def getSynapseDynamics(self, parameter_name, format='list', gather=True):
+        """
+        Get parameters of the dynamic synapses for all connections in this
+        Projection.
+        """
+        if gather:
+            logger.error("getSynapseDynamics() with gather=True not yet implemented")
+        return self.connection_manager.get(parameter_name, format, offset=(self.pre.first_id, self.post.first_id))
     
-    def saveConnections(self, filename, gather=False):
+    def saveConnections(self, filename, gather=True, compatible_output=True):
         """
         Save connections to file in a format suitable for reading in with a
         FromFileConnector.
         """
-        if gather == True:
-            raise Exception("saveConnections(gather=True) not yet supported")
+        if compatible_output:
+            fmt = "%s%s\t%s%s\t%s\t%s\n" % (self.pre.label, "%s", self.post.label,
+                                            "%s", "%g", "%g")
+        else:
+            fmt = "%s\t%s\t%s\t%s\n" % ("%d", "%d", "%g", "%g")
+        lines = []
+        if not compatible_output:
+            for c in self.connections:   
+                line = fmt  % (c.source, 
+                               c.target,
+                               c.weight,
+                               c.delay)
+                lines.append(line)
+        else:
+            for c in self.connections:   
+                line = fmt  % (self.pre.locate(c.source),
+                              self.post.locate(c.target),
+                              c.weight,
+                              c.delay)
+                line = line.replace('(','[').replace(')',']')
+                lines.append(line)
+        if gather == True and num_processes() > 1:
+            all_lines = { rank(): lines }
+            all_lines = recording.gather_dict(all_lines)
+            if rank() == 0:
+                lines = reduce(operator.add, all_lines.values())
         elif num_processes() > 1:
             filename += '.%d' % rank()
-        logging.debug("--- Projection[%s].__saveConnections__() ---" % self.label)
-        f = open(filename, 'w', DEFAULT_BUFFER_SIZE)
-        fmt = "%s%s\t%s%s\t%s\t%s\n" % (self.pre.label, "%s", self.post.label,
-                                        "%s", "%g", "%g")
-        for c in self.connections:     
-            line = fmt  % (self.pre.locate(c.source),
-                           self.post.locate(c.target),
-                           c.weight,
-                           c.delay)
-            line = line.replace('(','[').replace(')',']')
-            f.write(line)
-        f.close()
+        logger.debug("--- Projection[%s].__saveConnections__() ---" % self.label)
+        if gather == False or rank() == 0:
+            f = open(filename, 'w')
+            f.writelines(lines)
+            f.close()
     
     def printWeights(self, filename, format='list', gather=True):
         """Print synaptic weights to file. In the array format, zeros are printed
@@ -1459,72 +1519,6 @@ class Projection(object):
         else:
             return Template(template).substitute(context)
 
-
-# ==============================================================================
-#   Connection method classes
-# ==============================================================================
-
-class Connector(object):
-    """Base class for Connector classes."""
-    
-    def __init__(self, weights=0.0, delays=None, check_connections=False):
-        self.w_index = 0 # should probably use a generator
-        self.d_index = 0 # rather than storing these values
-        self.weights = weights
-        self.delays = delays
-        self.check_connections = check_connections
-        if delays is None:
-            self.delays = get_min_delay()
-    
-    def connect(self, projection):
-        """Connect all neurons in ``projection``"""
-        raise NotImplementedError()
-    
-    def get_weights(self, N, mask_local=None):
-        """
-        Returns the next N weight values
-        """
-        weights = next_n(self.weights, N, self.w_index, mask_local)
-        self.w_index += N
-        return weights
-    
-    def get_delays(self, N, mask_local=None):
-        """
-        Returns the next N delay values
-        """
-        delays = next_n(self.delays, N, self.d_index, mask_local)
-        self.d_index += N
-        if self.check_connections:
-            assert numpy.all(delays >= get_min_delay()), \
-            "Delay values must be greater than or equal to the minimum delay %g. The smallest delay is %g." % (get_min_delay(), delays.min())
-            assert numpy.all(delays <= get_max_delay()), \
-            "Delay values must be less than or equal to the maximum delay %s. The largest delay is %s" % (get_max_delay(), delays.max())                                                                                              
-        return delays
-    
-    def weights_iterator(self):
-        w = self.weights
-        if w is not None:
-            if hasattr(w, '__len__'): # w is an array
-                weights = w.__iter__()
-            else:
-                weights = ConstIter(w)
-        else: 
-            weights = ConstIter(1.0)
-        return weights
-    
-    def delays_iterator(self):
-        d = self.delays
-        min_delay = get_min_delay()
-        if d is not None:
-            if hasattr(d, '__len__'): # d is an array
-                if min(d) < min_delay:
-                    raise Exception("The array of delays contains one or more values that is smaller than the simulator minimum delay.")
-                delays = d.__iter__()
-            else:
-                delays = ConstIter(max((d, min_delay)))
-        else:
-            delays = ConstIter(min_delay)
-        return delays
         
 # ==============================================================================
 #   Synapse Dynamics classes
@@ -1573,7 +1567,7 @@ class STDPMechanism(object):
     """Specification of STDP models."""
     
     def __init__(self, timing_dependence=None, weight_dependence=None,
-                 voltage_dependence=None, dendritic_delay_fraction=1.0): #, model=None):
+                 voltage_dependence=None, dendritic_delay_fraction=1.0):
         """
         Create a new specification for an STDP mechanism, by combining a
         weight-dependence, a timing-dependence, and, optionally, a voltage-
@@ -1597,7 +1591,6 @@ class STDPMechanism(object):
         self.weight_dependence = weight_dependence
         self.voltage_dependence = voltage_dependence
         self.dendritic_delay_fraction = dendritic_delay_fraction
-        #self.model = model # see comment in Projection.__init__()
         
     def describe(self):
         """
