@@ -2,18 +2,25 @@
 """
 NEST v2 implementation of the PyNN API.
 
-$Id: __init__.py 637 2009-06-04 14:45:51Z apdavison $
+$Id: __init__.py 701 2010-02-02 15:05:38Z apdavison $
 """
 import nest
 from pyNN.nest import simulator
 from pyNN import common, recording, __doc__
 common.simulator = simulator
+recording.simulator = simulator
+
 from pyNN.random import *
-import numpy, os, shutil, logging, tempfile
+import numpy
+import os
+import shutil
+import logging
+import tempfile
 from pyNN.nest.cells import *
 from pyNN.nest.connectors import *
 from pyNN.nest.synapses import *
 from pyNN.nest.electrodes import *
+from pyNN.nest.recording import *
 
 Set = set
 
@@ -23,6 +30,8 @@ tempdirs       = []
 NEST_SYNAPSE_TYPES = ["cont_delay_synapse" ,"static_synapse", "stdp_pl_synapse_hom",
                       "stdp_synapse", "stdp_synapse_hom", "tsodyks_synapse"]
 
+logger = logging.getLogger("PyNN")
+
 # ==============================================================================
 #   Utility functions
 # ==============================================================================
@@ -57,7 +66,7 @@ def _discrepancy_due_to_rounding(parameters, output_values):
 #   Functions for simulation set-up and control
 # ==============================================================================
 
-def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False, **extra_params):
+def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, **extra_params):
     """
     Should be called at the very beginning of a script.
     extra_params contains any keyword arguments that are required by a given
@@ -65,7 +74,7 @@ def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False, **extra_para
     """
     global tempdir
     
-    common.setup(timestep, min_delay, max_delay, debug, **extra_params)
+    common.setup(timestep, min_delay, max_delay, **extra_params)
     
     if 'verbosity' in extra_params:
         nest_verbosity = extra_params['verbosity'].upper()
@@ -92,18 +101,20 @@ def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False, **extra_para
         rng_seeds_seed = extra_params.get('rng_seeds_seed') or 42
         rng = NumpyRNG(rng_seeds_seed)
         rng_seeds = (rng.rng.uniform(size=num_threads*num_processes())*100000).astype('int').tolist() 
-    logging.debug("rng_seeds = %s" % rng_seeds)
+    logger.debug("rng_seeds = %s" % rng_seeds)
     nest.SetKernelStatus({'local_num_threads': num_threads,
                           'rng_seeds'        : rng_seeds})
 
+    # set resolution
+    nest.SetKernelStatus({'resolution': timestep})
+
     # Set min_delay and max_delay for all synapse models
     for synapse_model in NEST_SYNAPSE_TYPES:
         nest.SetDefaults(synapse_model, {'delay' : min_delay,
                                          'min_delay': min_delay,
                                          'max_delay': max_delay})
-
-    # set resolution
-    nest.SetKernelStatus({'resolution': timestep})
+    simulator.reset()
+    
     return rank()
 
 def end(compatible_output=True):
@@ -112,7 +123,7 @@ def end(compatible_output=True):
     # And we postprocess the low level files opened by record()
     # and record_v() method
     for recorder in simulator.recorder_list:
-        recorder.write(gather=False, compatible_output=compatible_output)
+        recorder.write(gather=True, compatible_output=compatible_output)
     for tempdir in tempdirs:
         shutil.rmtree(tempdir)
     tempdirs = []
@@ -122,6 +133,8 @@ def run(simtime):
     simulator.run(simtime)
     return get_current_time()
 
+reset = common.reset
+
 # ==============================================================================
 #   Functions returning information about the simulation state
 # ==============================================================================
@@ -159,7 +172,6 @@ class Population(common.Population):
     An array of neurons all of the same type. `Population' is used as a generic
     term intended to include layers, columns, nuclei, etc., of cells.
     """
-    nPop = 0
 
     def __init__(self, dims, cellclass, cellparams=None, label=None):
         """
@@ -176,10 +188,6 @@ class Population(common.Population):
         label is an optional name for the population.
         """
         common.Population.__init__(self, dims, cellclass, cellparams, label)
-
-        # Should perhaps use "LayoutNetwork"?
-        if isinstance(cellclass, type) and issubclass(cellclass, common.StandardCellType):
-            self.celltype = cellclass(cellparams)
         
         self.all_cells, self._mask_local, self.first_id, self.last_id = simulator.create_cells(cellclass, cellparams, self.size, parent=self)
         self.local_cells = self.all_cells[self._mask_local]
@@ -190,12 +198,9 @@ class Population(common.Population):
             id.parent = self
         self.cell = self.all_cells # temporary alias, awaiting harmonization
         
-        if not self.label:
-            self.label = 'population%d' % Population.nPop
         self.recorders = {}
-        for variable in simulator.RECORDING_DEVICE_NAMES:
-            self.recorders[variable] = simulator.Recorder(variable, population=self)
-        Population.nPop += 1
+        for variable in RECORDING_DEVICE_NAMES:
+            self.recorders[variable] = Recorder(variable, population=self)
 
     def set(self, param, val=None):
         """
@@ -243,14 +248,14 @@ class Population(common.Population):
                 if key == 'v_init':
                     for cell in self.local_cells:
                         cell._v_init = value
-                    nest.SetStatus(self.local_cells, "V_m", val) # not correct, since could set v_init in the middle of a simulation
+                    nest.SetStatus(self.local_cells.tolist(), "V_m", val) # not correct, since could set v_init in the middle of a simulation
                 elif key in self.celltype.scaled_parameters():
                     translation = self.celltype.translations[key]
                     value = eval(translation['forward_transform'], globals(), {key:value})
-                    nest.SetStatus(self.local_cells,translation['translated_name'],value)
+                    nest.SetStatus(self.local_cells.tolist(), translation['translated_name'], value)
                 elif key in self.celltype.simple_parameters():
                     translation = self.celltype.translations[key]
-                    nest.SetStatus(self.local_cells, translation['translated_name'], value)
+                    nest.SetStatus(self.local_cells.tolist(), translation['translated_name'], value)
                 else:
                     for cell in self.local_cells:
                         cell.set_parameters(**{key:value})
@@ -298,28 +303,28 @@ class Population(common.Population):
         common.Population._record(self, variable, record_from, rng, to_file)
         nest.SetStatus(self.recorders[variable]._device, {'to_file': to_file, 'to_memory' : not to_file})
     
-    def meanSpikeCount(self, gather=True):
-        """
-        Returns the mean number of spikes per neuron.
-        """
-        # Routine to give an average firing rate over all the threads/nodes
-        # This is a rough approximation, because in fact each nodes is only multiplying 
-        # the frequency of the recorders by the number of processes. To do better, we need a MPI
-        # package to send informations to node 0. Nevertheless, it works for threaded mode
-        if gather:
-            node_list = range(nest.GetKernelStatus()["total_num_virtual_procs"])
-        else:
-            node_list = [rank()]
-        n_spikes  = 0
-        for node in node_list:
-            nest.sps(self.recorders['spikes']._device[0])
-            nest.sr("%d GetAddress %d append" %(self.recorders['spikes']._device[0], node))
-            nest.sr("GetStatus /n_events get")
-            n_spikes += nest.spp()
-        n_rec = len(self.recorders['spikes'].recorded)
-        if gather and num_processes()>1:
-            n_rec = recording.mpi_sum(n_rec)
-        return float(n_spikes)/n_rec
+    #def meanSpikeCount(self, gather=True):
+    #    """
+    #    Returns the mean number of spikes per neuron.
+    #    """
+    #    # Routine to give an average firing rate over all the threads/nodes
+    #    # This is a rough approximation, because in fact each nodes is only multiplying 
+    #    # the frequency of the recorders by the number of processes. To do better, we need a MPI
+    #    # package to send informations to node 0. Nevertheless, it works for threaded mode
+    #    if gather:
+    #        node_list = range(nest.GetKernelStatus()["total_num_virtual_procs"])
+    #    else:
+    #        node_list = [rank()]
+    #    n_spikes  = 0
+    #    for node in node_list:
+    #        nest.sps(self.recorders['spikes']._device[0])
+    #        nest.sr("%d GetAddress %d append" %(self.recorders['spikes']._device[0], node))
+    #        nest.sr("GetStatus /n_events get")
+    #        n_spikes += nest.spp()
+    #    n_rec = len(self.recorders['spikes'].recorded)
+    #    if gather and num_processes()>1:
+    #        n_rec = recording.mpi_sum(n_rec)
+    #    return float(n_spikes)/n_rec
 
     def getSubPopulation(self, cell_list, label=None):
         
@@ -375,10 +380,10 @@ class Projection(common.Projection):
         self.synapse_type = target or 'excitatory'
 
         if isinstance(self.long_term_plasticity_mechanism, Set):
-            logging.warning("Several STDP models are available for these connections:")
-            logging.warning(", ".join(model for model in self.long_term_plasticity_mechanism))
+            logger.warning("Several STDP models are available for these connections:")
+            logger.warning(", ".join(model for model in self.long_term_plasticity_mechanism))
             self.long_term_plasticity_mechanism = list(self.long_term_plasticity_mechanism)[0]
-            logging.warning("By default, %s is used" % self.long_term_plasticity_mechanism)
+            logger.warning("By default, %s is used" % self.long_term_plasticity_mechanism)
 
         if synapse_dynamics and synapse_dynamics.fast and synapse_dynamics.slow:
                 raise Exception("It is not currently possible to have both short-term and long-term plasticity at the same time with this simulator.")
@@ -409,16 +414,17 @@ class Projection(common.Projection):
             tau_minus = self._stdp_parameters.pop("tau_minus")
             # The following is a temporary workaround until the NEST guys stop renaming parameters!
             if 'tau_minus' in nest.GetStatus([self.post.local_cells[0]])[0]:
-                nest.SetStatus(self.post.local_cells, [{'tau_minus': tau_minus}])
+                nest.SetStatus(self.post.local_cells.tolist(), [{'tau_minus': tau_minus}])
             elif 'Tau_minus' in nest.GetStatus([self.post.local_cells[0]])[0]:
-                nest.SetStatus(self.post.local_cells, [{'Tau_minus': tau_minus}])
+                nest.SetStatus(self.post.local_cells.tolist(), [{'Tau_minus': tau_minus}])
             else:
                 raise Exception("Postsynaptic cell model does not support STDP.")
 
             synapse_defaults.update(self._stdp_parameters)
 
         nest.CopyModel(self._plasticity_model, self.plasticity_name, synapse_defaults)
-        self.connection_manager = simulator.ConnectionManager(self.plasticity_name, parent=self)
+        self.connection_manager = simulator.ConnectionManager(self.synapse_type,
+                                                              self.plasticity_name, parent=self)
         
         # Create connections
         method.connect(self)
@@ -426,21 +432,6 @@ class Projection(common.Projection):
         self.connections = self.connection_manager
 
 
-    # --- Methods for writing/reading information to/from file. ----------------
-
-    def _dump_connections(self):
-        """For debugging."""
-        print "Connections for Projection %s, connected with %s" % (self.label or '(un-labelled)',
-                                                                    self._method)
-        print "\tsource\ttarget\tport"
-        for src,tgt in zip(self._sources, self._targets):
-            connections = nest.FindConnections([src],[tgt],self.plasticity_name)
-            for port in connections['ports']:
-                print "\t%d\t%d\t%d" % (src, tgt, port)
-        print "Connection data for the presynaptic population (%s)" % self.pre.label
-        for src in self.pre.cell.flat:
-            print src, nest.GetConnections([src], self.plasticity_name)  
-
-
+Space = common.Space
 
 # ==============================================================================
