@@ -2,7 +2,7 @@
 """
 Standard cells for the neuron module.
 
-$Id: cells.py 626 2009-06-01 15:40:56Z apdavison $
+$Id: cells.py 704 2010-02-03 14:18:33Z apdavison $
 """
 
 from pyNN import common, cells
@@ -10,6 +10,7 @@ from neuron import h, nrn, hclass
 from math import pi
 import logging
 
+logger = logging.getLogger("PyNN")
 
 def _new_property(obj_hierarchy, attr_name):
     """
@@ -48,6 +49,7 @@ class SingleCompartmentNeuron(nrn.Section):
                 
         self.syn_type = syn_type
         self.syn_shape = syn_shape
+        self.v_init = v_init
         
         # insert synapses
         assert syn_type in ('current', 'conductance'), "syn_type must be either 'current' or 'conductance'. Actual value is %s" % syn_type
@@ -55,6 +57,9 @@ class SingleCompartmentNeuron(nrn.Section):
         synapse_model = StandardIF.synapse_models[syn_type][syn_shape]
         self.esyn = synapse_model(0.5, sec=self)
         self.isyn = synapse_model(0.5, sec=self)
+        if self.syn_type == 'conductance' and self.syn_shape == 'exp':
+            self.esyn_TM = h.tmgsyn(0.5, sec=self)
+            self.isyn_TM = h.tmgsyn(0.5, sec=self)
         
         # insert current source
         self.stim = h.IClamp(0.5, sec=self)
@@ -66,6 +71,7 @@ class SingleCompartmentNeuron(nrn.Section):
         self.spike_times = h.Vector(0)
         self.gsyn_trace = {}
         self.recording_time = 0
+        
 
     @property
     def excitatory(self):
@@ -74,6 +80,20 @@ class SingleCompartmentNeuron(nrn.Section):
     @property
     def inhibitory(self):
         return self.isyn
+    
+    @property
+    def excitatory_TM(self):
+        if hasattr(self, 'esyn_TM'):
+            return self.esyn_TM
+        else:
+            return None
+
+    @property
+    def inhibitory_TM(self):
+        if hasattr(self, 'isyn_TM'):
+            return self.isyn_TM
+        else:
+            return None
 
     def area(self):
         """Membrane area in µm²"""
@@ -81,10 +101,38 @@ class SingleCompartmentNeuron(nrn.Section):
 
     c_m      = _new_property('seg', 'cm')
     i_offset = _new_property('stim', 'amp')
-    tau_e    = _new_property('esyn', 'tau')
-    tau_i    = _new_property('isyn', 'tau')
-    e_e      = _new_property('esyn', 'e')
-    e_i      = _new_property('isyn', 'e')
+    
+    def _get_tau_e(self):
+        return self.esyn.tau
+    def _set_tau_e(self, value):
+        self.esyn.tau = value
+        if hasattr(self, 'esyn_TM'):
+            self.esyn_TM.tau = value
+    tau_e = property(fget=_get_tau_e, fset=_set_tau_e)
+    
+    def _get_tau_i(self):
+        return self.isyn.tau
+    def _set_tau_i(self, value):
+        self.isyn.tau = value
+        if hasattr(self, 'isyn_TM'):
+            self.isyn_TM.tau = value
+    tau_i = property(fget=_get_tau_i, fset=_set_tau_i)
+    
+    def _get_e_e(self):
+        return self.esyn.e
+    def _set_e_e(self, value):
+        self.esyn.e = value
+        if hasattr(self, 'esyn_TM'):
+            self.esyn_TM.e = value
+    e_e = property(fget=_get_e_e, fset=_set_e_e)
+    
+    def _get_e_i(self):
+        return self.isyn.e
+    def _set_e_i(self, value):
+        self.isyn.e = value
+        if hasattr(self, 'isyn_TM'):
+            self.isyn_TM.e = value
+    e_i = property(fget=_get_e_i, fset=_set_e_i)
     
     def record(self, active):
         if active:
@@ -106,6 +154,8 @@ class SingleCompartmentNeuron(nrn.Section):
                 self.record_times = None
     
     def record_gsyn(self, syn_name, active):
+        # how to deal with static and T-M synapses?
+        # record both and sum?
         if active:
             self.gsyn_trace[syn_name] = h.Vector()
             self.gsyn_trace[syn_name].record(getattr(self, syn_name)._ref_g)
@@ -122,36 +172,26 @@ class SingleCompartmentNeuron(nrn.Section):
     def memb_init(self, v_init=None):
         if v_init:
             self.v_init = v_init
-        self.seg.v = self.v_init
+        assert self.v_init is not None, "cell is a %s" % self.__class__.__name__
+        for seg in self:
+            seg.v = self.v_init
+        #self.seg.v = self.v_init
 
-    def use_Tsodyks_Markram_synapses(self, ei, U, tau_rec, tau_facil, u0):
+    def set_Tsodyks_Markram_synapses(self, ei, U, tau_rec, tau_facil, u0):
         if self.syn_type == 'current':
             raise Exception("Tsodyks-Markram mechanism only available for conductance-based synapses.")
         elif self.syn_shape == 'alpha':
             raise Exception("Tsodyks-Markram mechanism not available for alpha-function-shaped synapses.")
         elif ei == 'excitatory':
-            tau_syn = self.tau_e
-            e_syn = self.e_e
-            self.esyn = h.tmgsyn(0.5, sec=self)
-            self.esyn.tau = tau_syn
-            self.esyn.e = e_syn
-            syn = self.esyn
+            syn = self.esyn_TM
         elif ei == 'inhibitory':
-            tau_syn = self.tau_i
-            e_syn = self.e_i
-            self.isyn = h.tmgsyn(0.5, sec=self)
-            self.isyn.tau = tau_syn
-            self.isyn.e = e_syn
-            syn = self.isyn
+            syn = self.isyn_TM
         else:
             raise Exception("Tsodyks-Markram mechanism not yet implemented for user-defined synapse types. ei = %s" % ei)
         syn.U = U
         syn.tau_rec = tau_rec
         syn.tau_facil = tau_facil
         syn.u0 = u0
-        # change the mechanism that is being recorded
-        if ei in self.gsyn_trace and self.gsyn_trace[ei] is not None:
-            self.gsyn_trace[ei].record(syn._ref_g)
 
     def set_parameters(self, param_dict):
         for name in self.parameter_names:
@@ -196,13 +236,12 @@ class StandardIF(LeakySingleCompartmentNeuron):
     def __init__(self, syn_type, syn_shape, tau_m=20, c_m=1.0, v_rest=-65,
                  v_thresh=-55, t_refrac=2, i_offset=0, v_reset=None,
                  v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70):
-        LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m, c_m, v_rest,
-                                         i_offset, v_init, tau_e, tau_i, e_e, e_i)
-        if v_reset is None:
-            v_reset = v_rest
         if v_init is None:
             v_init = v_rest
-        
+        if v_reset is None:
+            v_reset = v_rest
+        LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m, c_m, v_rest,
+                                              i_offset, v_init, tau_e, tau_i, e_e, e_i)       
         # insert spike reset mechanism
         self.spike_reset = h.ResetRefrac(0.5, sec=self)
         self.spike_reset.vspike = 40 # (mV) spike height
@@ -214,12 +253,13 @@ class StandardIF(LeakySingleCompartmentNeuron):
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
+        
 
     v_thresh = _new_property('spike_reset', 'vthresh')
     v_reset  = _new_property('spike_reset', 'vreset')
     t_refrac = _new_property('spike_reset', 'trefrac')
-    
-    
+
+
 class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     """docstring"""
     
@@ -228,52 +268,50 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
                  v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70,
                  v_spike=0.0, v_reset=-70.6, A=4.0, B=0.0805, tau_w=144.0,
                  w_init=0.0, delta=2.0):
-        LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m, c_m, v_rest,
-                                         i_offset, v_init,
-                                         tau_e, tau_i, e_e, e_i)
         if v_init is None:
             v_init = v_rest
+        LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m,
+                                              c_m, v_rest, i_offset, v_init,
+                                              tau_e, tau_i, e_e, e_i)
     
         # insert Brette-Gerstner spike mechanism
-        self.insert('IF_BG5')
-        self.seg.IF_BG5.surf = self.area()
+        self.adexp = h.AdExpIF(0.5, sec=self)
         self.source = self.seg._ref_v
         
         self.parameter_names = ['c_m', 'tau_m', 'v_rest', 'v_thresh', 't_refrac',
                                 'i_offset', 'v_reset', 'v_init', 'tau_e', 'tau_i',
-                                'v_thresh', 'A', 'B', 'tau_w', 'delta', 'w_init',
-                                'v_spike']
+                                'A', 'B', 'tau_w', 'delta', 'w_init', 'v_spike']
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
     
-    v_thresh = _new_property('seg.IF_BG5', 'Vtr')
-    v_reset  = _new_property('seg.IF_BG5', 'Vbot')
-    t_refrac = _new_property('seg.IF_BG5', 'Ref')
-    B        = _new_property('seg.IF_BG5',  'b')
-    A        = _new_property('seg.IF_BG5',  'a')
-    # using 'A' because for some bizarre reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"   
-    tau_w    = _new_property('seg.IF_BG5',  'tau_w')
-    delta    = _new_property('seg.IF_BG5',  'delta')
-    w_init   = _new_property('seg.IF_BG5',  'w_init') # w_init not defined in .mod file - needs to be
-    v_init   = _new_property('seg',  'v') #?? something involving FInitialize
+    v_thresh = _new_property('adexp', 'vthresh')
+    v_reset  = _new_property('adexp', 'vreset')
+    t_refrac = _new_property('adexp', 'trefrac')
+    B        = _new_property('adexp',  'b')
+    A        = _new_property('adexp',  'a')
+    ## using 'A' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"   
+    tau_w    = _new_property('adexp',  'tauw')
+    delta    = _new_property('adexp',  'delta')
+    w_init   = _new_property('adexp',  'winit') # w_init not defined in .mod file - needs to be
+    #v_init   = _new_property('seg',  'v') #?? something involving FInitialize
     
     def __set_v_spike(self, value):
-        self.seg.IF_BG5.Vspike = value
-        self.seg.IF_BG5.Vtop = value + 10.0
+        self.adexp.vspike = value
+        self.adexp.vpeak = value + 10.0
     def __get_v_spike(self):
-        return self.seg.IF_BG5.Vspike
+        return self.adexp.vspike
     v_spike = property(fget=__get_v_spike, fset=__set_v_spike)
     
     def __set_tau_m(self, value):
         self.seg.pas.g = 1e-3*self.seg.cm/value # cm(nF)/tau_m(ms) = G(uS) = 1e-6G(S). Divide by area (1e-3) to get factor of 1e-3
-        self.seg.IF_BG5.GL = self.seg.pas.g
+        self.adexp.GL = self.seg.pas.g * self.area() * 1e-2 # S/cm2 to uS
     def __get_tau_m(self):
         return 1e-3*self.seg.cm/self.seg.pas.g
-
+    
     def __set_v_rest(self, value):
         self.seg.pas.e = value
-        self.seg.IF_BG5.EL = value
+        self.adexp.EL = value
     def __get_v_rest(self):
         return self.seg.pas.e
     tau_m  = property(fget=__get_tau_m, fset=__set_tau_m)   
@@ -281,9 +319,13 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     
     def record(self, active):
         if active:
-            rec = h.NetCon(self.source, None, sec=self)
-            rec.record(self.spike_times)
-    
+            self.rec = h.NetCon(self.source, None,
+                                self.get_threshold(), 0.0, 0.0,
+                                sec=self)
+            self.rec.record(self.spike_times)
+
+    def get_threshold(self):
+        return self.adexp.vspike
 
 class SingleCompartmentTraub(SingleCompartmentNeuron):
     
@@ -291,10 +333,10 @@ class SingleCompartmentTraub(SingleCompartmentNeuron):
                  i_offset=0, v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70,
                  gbar_Na=20000, gbar_K=6000, g_leak=10, ena=50,
                  ek=-90, v_offset=-63):
-        SingleCompartmentNeuron.__init__(self, syn_type, syn_shape, c_m, i_offset,
-                                         v_init, tau_e, tau_i, e_e, e_i)
         if v_init is None:
             v_init = v_rest
+        SingleCompartmentNeuron.__init__(self, syn_type, syn_shape, c_m, i_offset,
+                                         v_init, tau_e, tau_i, e_e, e_i)
         self.source = self.seg._ref_v
         self.insert('k_ion')
         self.insert('na_ion')
@@ -305,24 +347,41 @@ class SingleCompartmentTraub(SingleCompartmentNeuron):
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
-        
+
+    # not sure ena and ek are handled correctly
+    
     e_leak   = _new_property('seg.hh_traub', 'el')
     v_offset = _new_property('seg.hh_traub', 'vT')
     gbar_Na  = _new_property('seg.hh_traub', 'gnabar')
     gbar_K   = _new_property('seg.hh_traub', 'gkbar')
     g_leak   = _new_property('seg.hh_traub', 'gl')
     
+    def get_threshold(self):
+        return 10.0
+    
+    
 class RandomSpikeSource(hclass(h.NetStimFD)):
     
-    parameter_names = ('start', 'interval', 'duration')
+    parameter_names = ('start', '_interval', 'duration')
     
-    def __init__(self, start=0, interval=1e12, duration=0):
+    def __init__(self, start=0, _interval=1e12, duration=0):
         self.start = start
-        self.interval = interval
+        self.interval = _interval
         self.duration = duration
         self.noise = 1
         self.spike_times = h.Vector(0)
         self.source = self
+        self.switch = h.NetCon(None, self)
+
+    def _set_interval(self, value):
+        self.switch.weight[0] = -1
+        self.switch.event(h.t+1e-12, 0)
+        self.interval = value
+        self.switch.weight[0] = 1
+        self.switch.event(h.t+2e-12, 1)
+    def _get_interval(self):
+        return self.interval
+    _interval = property(fget=_get_interval, fset=_set_interval)
 
     def record(self, active):
         if active:
@@ -339,7 +398,10 @@ class VectorSpikeSource(hclass(h.VecStim)):
         self.source = self
             
     def _set_spike_times(self, spike_times):
-        self._spike_times = h.Vector(spike_times)
+        try:
+            self._spike_times = h.Vector(spike_times)
+        except RuntimeError:
+            raise common.InvalidParameterValueError("spike_times must be an array of floats")
         self.play(self._spike_times)
             
     def _get_spike_times(self):
@@ -520,7 +582,7 @@ class SpikeSourcePoisson(cells.SpikeSourcePoisson):
 
     translations = common.build_translations(
         ('start',    'start'),
-        ('rate',     'interval',  "1000.0/rate",  "1000.0/interval"),
+        ('rate',     '_interval',  "1000.0/rate",  "1000.0/_interval"),
         ('duration', 'duration'),
     )
     model = RandomSpikeSource
