@@ -24,11 +24,11 @@ Attributes:
 All other functions and classes are private, and should not be used by other
 modules.
     
-$Id: simulator.py 628 2009-06-02 13:59:46Z apdavison $
+$Id: simulator.py 702 2010-02-03 10:39:32Z apdavison $
 """
 
 from pyNN import __path__ as pyNN_path
-from pyNN import common, recording, random
+from pyNN import common, random
 import platform
 import logging
 import numpy
@@ -39,7 +39,8 @@ from neuron import h
 nrn_dll_loaded = []
 recorder_list = []
 connection_managers = []
-
+gid_sources = []
+logger = logging.getLogger("PyNN")
 
 # --- Internal NEURON functionality -------------------------------------------- 
 
@@ -63,12 +64,20 @@ def is_point_process(obj):
 
 def register_gid(gid, source, section=None):
     """Register a global ID with the global `ParallelContext` instance."""
-    state.parallel_context.set_gid2node(gid, state.mpi_rank)  # assign the gid to this node
+    ###print "registering gid %s to %s (section=%s)" % (gid, source, section)
+    state.parallel_context.set_gid2node(gid, state.mpi_rank) # assign the gid to this node
     if is_point_process(source):
         nc = h.NetCon(source, None)                          # } associate the cell spike source
     else:
         nc = h.NetCon(source, None, sec=section)
     state.parallel_context.cell(gid, nc)              # } with the gid (using a temporary NetCon)
+    gid_sources.append(source) # gid_clear (in _State.reset()) will cause a
+                               # segmentation fault if any of the sources
+                               # registered using pc.cell() no longer exist, so
+                               # we keep a reference to all sources in the
+                               # global gid_sources list. It would be nicer to
+                               # be able to unregister a gid and have a __del__
+                               # method in ID, but this will do for now.
 
 def nativeRNG_pick(n, rng, distribution='uniform', parameters=[0,1]):
     """
@@ -105,15 +114,10 @@ class _Initializer(object):
         Create an `FinitializeHandler` object in Hoc, which will call the
         `_initialize()` method when NEURON is initialized.
         """
-        self.cell_list = []
-        self.population_list = []
         h('objref initializer')
         h.initializer = self
-        self.fih = h.FInitializeHandler("initializer._initialize()")
-    
-    #def __call__(self):
-    #    """This is to make the Initializer a Singleton."""
-    #    return self
+        self.fih = h.FInitializeHandler(0, "initializer._initialize()")
+        self.clear()
     
     def register(self, *items):
         """
@@ -130,7 +134,7 @@ class _Initializer(object):
     
     def _initialize(self):
         """Call `memb_init()` for all registered cell objects."""
-        logging.info("Initializing membrane potential of %d cells and %d Populations." % \
+        logger.info("Initializing membrane potential of %d cells and %d Populations." % \
                      (len(self.cell_list), len(self.population_list)))
         for cell in self.cell_list:
             cell._cell.memb_init()
@@ -138,6 +142,10 @@ class _Initializer(object):
             for cell in population:
                 cell._cell.memb_init()
 
+    def clear(self):
+        self.cell_list = []
+        self.population_list = []
+        
 
 # --- For implementation of get_time_step() and similar functions --------------
 
@@ -146,25 +154,35 @@ class _State(object):
     
     def __init__(self):
         """Initialize the simulator."""
-        self.gid_counter = 0
-        self.running = False
-        self.initialized = False
         h('min_delay = 0')
         h('tstop = 0')
+        h('steps_per_ms = 1/dt')
         self.parallel_context = h.ParallelContext()
         self.parallel_context.spike_compress(1,0)
         self.num_processes = int(self.parallel_context.nhost())
         self.mpi_rank = int(self.parallel_context.id())
         self.cvode = h.CVode()
-        self.max_delay = 1e12
         h('objref plastic_connections')
-        h.plastic_connections = []
+        self.clear()
     
     t = h_property('t')
-    dt = h_property('dt')
-    tstop = h_property('tstop')         # } do these really need to be stored in hoc?
-    min_delay = h_property('min_delay') # }
+    def __get_dt(self):
+        return h.dt
+    def __set_dt(self, dt):
+        h.steps_per_ms = 1.0/dt
+        h.dt = dt
+    dt = property(fget=__get_dt, fset=__set_dt)
+    tstop = h_property('tstop')         # } these are stored in hoc so that we
+    min_delay = h_property('min_delay') # } can interact with the GUI
 
+    def clear(self):
+        global connection_managers, gid_sources
+        self.parallel_context.gid_clear()
+        gid_sources = []
+        self.gid_counter = 0
+        self.running = False
+        h.plastic_connections = []
+        connection_managers = []
 
 def reset():
     """Reset the state of the current network to time t = 0."""
@@ -179,20 +197,20 @@ def run(simtime):
         local_minimum_delay = state.parallel_context.set_maxstep(10)
         h.finitialize()
         state.tstop = 0
-        logging.debug("local_minimum_delay on host #%d = %g" % (state.mpi_rank, local_minimum_delay))
+        logger.debug("local_minimum_delay on host #%d = %g" % (state.mpi_rank, local_minimum_delay))
         if state.num_processes > 1:
             assert local_minimum_delay >= state.min_delay,\
                    "There are connections with delays (%g) shorter than the minimum delay (%g)" % (local_minimum_delay, state.min_delay)
     state.tstop += simtime
-    logging.info("Running the simulation for %d ms" % simtime)
+    logger.info("Running the simulation for %d ms" % simtime)
     state.parallel_context.psolve(state.tstop)
 
-def finalize(quit=True):
+def finalize(quit=False):
     """Finish using NEURON."""
     state.parallel_context.runworker()
     state.parallel_context.done()
     if quit:
-        logging.info("Finishing up with NEURON.")
+        logger.info("Finishing up with NEURON.")
         h.quit()
 
 
@@ -220,6 +238,8 @@ class ID(int, common.IDMixin):
         gid = int(self)
         self._cell = cell_model(**cell_parameters)          # create the cell object
         register_gid(gid, self._cell.source, section=self._cell) # not adequate for multi-compartmental cells
+        if hasattr(self._cell, "get_threshold"):            # this is not adequate, since the threshold may be changed after cell creation
+            state.parallel_context.threshold(int(self), self._cell.get_threshold()) # the problem is that self._cell does not know its own gid
         
     def get_native_parameters(self):
         """Return a dictionary of parameters for the NEURON cell model."""
@@ -232,139 +252,6 @@ class ID(int, common.IDMixin):
         """Set parameters of the NEURON cell model from a dictionary."""
         for name, val in parameters.items():
             setattr(self._cell, name, val)
-
-
-# --- For implementation of record_X()/get_X()/print_X() -----------------------
-
-class Recorder(object):
-    """Encapsulates data and functions related to recording model variables."""
-    
-    numpy1_1_formats = {'spikes': "%g\t%d",
-                        'v': "%g\t%g\t%d",
-                        'gsyn': "%g\t%g\t%g\t%d"}
-    numpy1_0_formats = {'spikes': "%g", # only later versions of numpy support different
-                        'v': "%g",      # formats for different columns
-                        'gsyn': "%g"}
-    formats = {'spikes': 'id t',
-               'v': 'id t v',
-               'gsyn': 'id t ge gi'}
-    
-    def __init__(self, variable, population=None, file=None):
-        """
-        Create a recorder.
-        
-        `variable` -- "spikes", "v" or "gsyn"
-        `population` -- the Population instance which is being recorded by the
-                        recorder (optional)
-        `file` -- one of:
-            - a file-name,
-            - `None` (write to a temporary file)
-            - `False` (write to memory).
-        """
-        self.variable = variable
-        self.filename = file or None
-        self.population = population # needed for writing header information
-        self.recorded = set([])
-        
-    def record(self, ids):
-        """Add the cells in `ids` to the set of recorded cells."""
-        logging.debug('Recorder.record(%s)', str(ids))
-        if self.population:
-            ids = set([id for id in ids if id in self.population.local_cells])
-        else:
-            ids = set([id for id in ids if id.local])
-        new_ids = list( ids.difference(self.recorded) )
-        
-        self.recorded = self.recorded.union(ids)
-        logging.debug('Recorder.recorded = %s' % self.recorded)
-        if self.variable == 'spikes':
-            for id in new_ids:
-                id._cell.record(1)
-        elif self.variable == 'v':
-            for id in new_ids:
-                id._cell.record_v(1)
-        elif self.variable == 'gsyn':
-            for id in new_ids:
-                id._cell.record_gsyn("excitatory", 1)
-                id._cell.record_gsyn("inhibitory", 1)
-        else:
-            raise Exception("Recording of %s not implemented." % self.variable)
-        
-    def get(self, gather=False, compatible_output=True, offset=None):
-        """Return the recorded data as a Numpy array."""
-        # compatible_output is not used, but is needed for compatibility with the nest2 module.
-        # Does nest2 really need it?
-        if offset is None:
-            if self.population:
-                offset = self.population.first_id
-            else:
-                offset = 0
-                
-        if self.variable == 'spikes':
-            data = numpy.empty((0,2))
-            for id in self.recorded:
-                spikes = numpy.array(id._cell.spike_times)
-                spikes = spikes[spikes<=state.t+1e-9]
-                if len(spikes) > 0:    
-                    new_data = numpy.array([numpy.ones(spikes.shape)*(id-offset), spikes]).T
-                    data = numpy.concatenate((data, new_data))
-        elif self.variable == 'v':
-            data = numpy.empty((0,3))
-            for id in self.recorded:
-                v = numpy.array(id._cell.vtrace)  
-                t = numpy.array(id._cell.record_times)
-                #new_data = numpy.array([t, v, numpy.ones(v.shape)*id]).T
-                #new_data = numpy.array([numpy.ones(v.shape)*id, t, v]).T                
-                new_data = numpy.array([numpy.ones(v.shape)*(id-offset), t, v]).T
-                data = numpy.concatenate((data, new_data))
-        elif self.variable == 'gsyn':
-            data = numpy.empty((0,4))
-            for id in self.recorded:
-                ge = numpy.array(id._cell.gsyn_trace['excitatory'])
-                gi = numpy.array(id._cell.gsyn_trace['inhibitory'])
-                t = numpy.array(id._cell.record_times)             
-                new_data = numpy.array([numpy.ones(ge.shape)*(id-offset), t, ge, gi]).T
-                data = numpy.concatenate((data, new_data))
-        else:
-            raise Exception("Recording of %s not implemented." % self.variable)
-        if gather and state.num_processes > 1:
-            data = recording.gather(data)
-        return data
-    
-    def write(self, file=None, gather=False, compatible_output=True):
-        """Write recorded data to file."""
-        data = self.get(gather)
-        filename = file or self.filename
-        #recording.rename_existing(filename) # commented out because it causes problems when running with mpirun and a shared filesystem. Probably a timing problem
-        try:
-            numpy.savetxt(filename, data, Recorder.numpy1_0_formats[self.variable], delimiter='\t')
-        except AttributeError, errmsg:
-            # we assume the error is due to the lack of savetxt in older versions of numpy and
-            # so provide a cut-down version of that function
-            f = open(filename, 'w')
-            fmt = Recorder.numpy1_0_formats[self.variable]
-            for row in data:
-                f.write('\t'.join([fmt%val for val in row]) + '\n')
-            f.close()
-        if compatible_output and state.mpi_rank==0: # would be better to distribute this step
-            recording.write_compatible_output(filename, filename, self.variable,
-                                              Recorder.formats[self.variable],
-                                              self.population, state.dt)
-        
-    def count(self, gather=False):
-        """
-        Return the number of data points for each cell, as a dict. This is mainly
-        useful for spike counts or for variable-time-step integration methods.
-        """
-        N = {}
-        if self.variable == 'spikes':
-            for id in self.recorded:
-                N[id] = id._cell.spike_times.size()
-        else:
-            raise Exception("Only implemented for spikes.")
-        if gather and state.num_processes > 1:
-            N = recording.gather_dict(N)
-        return N
         
 
 # --- For implementation of create() and Population.__init__() -----------------
@@ -496,6 +383,39 @@ class Connection(object):
     weight = property(_get_weight, _set_weight)
     delay = property(_get_delay, _set_delay)
 
+def generate_synapse_property(name):
+    def _get(self):
+        synapse = self.nc.syn()
+        if hasattr(synapse, name):
+            return getattr(synapse, name)
+        else:
+            raise Exception("synapse type does not have an attribute '%s'" % name)
+    def _set(self, val):
+        synapse = self.nc.syn()
+        if hasattr(synapse, name):
+            setattr(synapse, name, val)
+        else:
+            raise Exception("synapse type does not have an attribute 'U'")
+    return property(_get, _set)
+setattr(Connection, 'U', generate_synapse_property('U'))
+setattr(Connection, 'tau_rec', generate_synapse_property('tau_rec'))
+setattr(Connection, 'tau_facil', generate_synapse_property('tau_facil'))
+setattr(Connection, 'u0', generate_synapse_property('u0'))
+
+def generate_stdp_property(name):
+    def _get(self):
+        return getattr(self.weight_adjuster, name)
+    def _set(self, val):
+        setattr(self.weight_adjuster, name, val)
+    return property(_get, _set)
+setattr(Connection, 'w_max', generate_stdp_property('wmax'))
+setattr(Connection, 'w_min', generate_stdp_property('wmin'))
+setattr(Connection, 'A_plus', generate_stdp_property('aLTP'))
+setattr(Connection, 'A_minus', generate_stdp_property('aLTD'))
+setattr(Connection, 'tau_plus', generate_stdp_property('tauLTP'))
+setattr(Connection, 'tau_minus', generate_stdp_property('tauLTD'))
+
+
 
 class ConnectionManager(object):
     """
@@ -503,16 +423,18 @@ class ConnectionManager(object):
     accessing individual connections.
     """
 
-    def __init__(self, synapse_model=None, parent=None):
+    def __init__(self, synapse_type, synapse_model=None, parent=None):
         """
         Create a new ConnectionManager.
         
-        `synapse_model` -- not used. Present for consistency with other simulators.
+        `synapse_model` -- either None or 'Tsodyks-Markram'.
         `parent` -- the parent `Projection`, if any.
         """
         global connection_managers
         self.connections = []
         self.parent = parent
+        self.synapse_type = synapse_type
+        self.synapse_model = synapse_model
         connection_managers.append(self)
         
     def __getitem__(self, i):
@@ -527,7 +449,7 @@ class ConnectionManager(object):
         """Return an iterator over all connections on the local MPI node."""
         return iter(self.connections)
     
-    def connect(self, source, targets, weights, delays, synapse_type):
+    def connect(self, source, targets, weights, delays):
         """
         Connect a neuron to one or more other neurons with a static connection.
         
@@ -537,11 +459,6 @@ class ConnectionManager(object):
                      Must have the same length as `targets`.
         `delays`  -- a list/1D array of connection delays, or a single delay.
                      Must have the same length as `targets`.
-        `synapse_type` -- a string identifying the synapse to connect to. Should
-                          be "excitatory", "inhibitory", or the name of any
-                          other attribute of the post-synaptic cell that refers
-                          to a synaptic mechanism. May be `None`, which is
-                          treated the same as "excitatory".
         """
         if not isinstance(source, int) or source > state.gid_counter or source < 0:
             errmsg = "Invalid source ID: %s (gid_counter=%d)" % (source, state.gid_counter)
@@ -560,12 +477,15 @@ class ConnectionManager(object):
         assert len(targets) == len(weights) == len(delays), "%s %s %s" % (len(targets),len(weights),len(delays))
         for target, weight, delay in zip(targets, weights, delays):
             if target.local:
-                if synapse_type is None:
-                    synapse_type = weight>=0 and 'excitatory' or 'inhibitory' 
-                synapse_object = getattr(target._cell, synapse_type)
+                if self.synapse_type is None:
+                    self.synapse_type = weight>=0 and 'excitatory' or 'inhibitory'
+                if self.synapse_model == 'Tsodyks-Markram' and 'TM' not in self.synapse_type:
+                    self.synapse_type += '_TM'
+                synapse_object = getattr(target._cell, self.synapse_type)
                 nc = state.parallel_context.gid_connect(int(source), synapse_object)
                 nc.weight[0] = weight
                 nc.delay  = delay
+                # nc.threshold is supposed to be set by ParallelContext.threshold, called in _build_cell(), above, but this hasn't been tested
                 self.connections.append(Connection(source, target, nc))
 
     def get(self, parameter_name, format, offset=(0,0)):
@@ -608,11 +528,25 @@ class ConnectionManager(object):
         
         `name`  -- attribute name
         `value` -- the attribute numeric value, or a list/1D array of such
-                   values of the same length as the number of local connections.
+                   values of the same length as the number of local connections,
+                   or a 2D array with the same dimensions as the connectivity
+                   matrix (as returned by `get(format='array')`).
         """
         if common.is_number(value):
             for c in self:
                 setattr(c, name, value)
+        elif isinstance(value, numpy.ndarray) and len(value.shape) == 2:
+            offset = (self.parent.pre.first_id, self.parent.post.first_id)
+            for c in self.connections:
+                addr = (c.source-offset[0], c.target-offset[1])
+                try:
+                    val = value[addr]
+                except IndexError, e:
+                    raise IndexError("%s. addr=%s" % (e, addr))
+                if numpy.isnan(val):
+                    raise Exception("Array contains no value for synapse from %d to %d" % (c.source, c.target))
+                else:
+                    setattr(c, name, val)
         elif common.is_listlike(value):
             for c,val in zip(self.connections, value):
                 setattr(c, name, val)
