@@ -7,16 +7,17 @@ pypcsim implementation of the PyNN API.
     Andrew Davison   davison@unic.cnrs-gif.fr
         
     December 2006-
-$Id: __init__.py 635 2009-06-04 14:15:18Z apdavison $
+$Id: __init__.py 701 2010-02-02 15:05:38Z apdavison $
 """
-__version__ = "$Revision: 635 $"
+__version__ = "$Revision: 701 $"
 
 import sys
 
 import pyNN.random
-from pyNN import common, __doc__
+from pyNN import common, recording, __doc__
 from pyNN.pcsim import simulator
 common.simulator = simulator
+recording.simulator = simulator
 import os.path
 import types
 import sys
@@ -26,6 +27,7 @@ from pyNN.pcsim.cells import *
 from pyNN.pcsim.connectors import *
 from pyNN.pcsim.synapses import *
 from pyNN.pcsim.electrodes import *
+from pyNN.pcsim.recording import *
 
 try:
     import tables
@@ -141,7 +143,7 @@ class WDManager(object):
 #   Functions for simulation set-up and control
 # ==============================================================================
 
-def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False, **extra_params):
+def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, **extra_params):
     """
     Should be called at the very beginning of a script.
     extra_params contains any keyword arguments that are required by a given
@@ -180,13 +182,13 @@ def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, debug=False, **extra_para
     #simulator.state.dt = timestep # seems to mess up the net object
     simulator.state.min_delay = min_delay
     simulator.state.max_delay = max_delay
-    common.setup(timestep, min_delay, max_delay, debug, **extra_params)
+    common.setup(timestep, min_delay, max_delay, **extra_params)
     return simulator.net.mpi_rank()
 
 def end(compatible_output=True):
     """Do any necessary cleaning up before exiting."""
     for recorder in simulator.recorder_list:
-        recorder.write(gather=False, compatible_output=compatible_output)
+        recorder.write(gather=True, compatible_output=compatible_output)
      
 
 def run(simtime):
@@ -195,6 +197,8 @@ def run(simtime):
     simulator.net.advance(int(simtime / simulator.state.dt ))
     return simulator.state.t
 
+reset = common.reset
+
 get_current_time = common.get_current_time
 get_time_step = common.get_time_step
 get_min_delay = common.get_min_delay
@@ -292,7 +296,6 @@ class Population(common.Population):
     id = p[address]
     address = p.locate(id)
     """
-    nPop = 0
     
     def __init__(self, dims, cellclass, cellparams=None, label=None):
         """
@@ -361,15 +364,9 @@ class Population(common.Population):
         self._mask_local = self._mask_local.reshape(self.dim)
         self.cell = self.all_cells # temporary, awaiting harmonisation
         
-        self.recorders = {'spikes': simulator.Recorder('spikes', population=self),
-                          'v': simulator.Recorder('v', population=self),
-                          'gsyn': simulator.Recorder('gsyn', population=self)}
-        
-        
-        if not self.label:
-            self.label = 'population%d' % Population.nPop         
-        Population.nPop += 1
-        
+        self.recorders = {'spikes': Recorder('spikes', population=self),
+                          'v': Recorder('v', population=self),
+                          'gsyn': Recorder('gsyn', population=self)}
         
     ##def __getitem__(self, addr):
     ##    """Return a representation of the cell with coordinates given by addr,
@@ -563,7 +560,7 @@ class Projection(common.Projection, WDManager):
         common.Projection.__init__(self, presynaptic_population, postsynaptic_population,
                                    method, source, target,
                                    synapse_dynamics, label, rng)
-        self.is_conductance = "cond" in self.post.celltype.__class__.__name__
+        self.is_conductance = self.post.celltype.__class__.conductance_based
         self.synapse_shape = ("alpha" in self.post.celltype.__class__.__name__) and "alpha" or "exp"
         
         ### Determine connection decider
@@ -701,8 +698,9 @@ class Projection(common.Projection, WDManager):
         ##    assert len(delay) == len(self), "Weight array does not have the same number of elements as the Projection %d != %d" % (len(weight),len(self))
         ##    self.setDelays(delay)
 
-        self.synapse_type = self.syn_factory #target or 'excitatory'
-        self.connection_manager = simulator.ConnectionManager(parent=self)
+        ##self.synapse_type = self.syn_factory #target or 'excitatory'
+        self.synapse_type = target or 'excitatory'
+        self.connection_manager = simulator.ConnectionManager(self.syn_factory, parent=self)
         self.connections = self.connection_manager
         method.connect(self)
         Projection.nProj += 1        
@@ -823,6 +821,7 @@ class Projection(common.Projection, WDManager):
     ##    f.close()
     
 
+Space = common.Space
 
 # ==============================================================================
 
