@@ -1,7 +1,7 @@
 """
 Standard cells for pcsim
 
-$Id: cells.py 626 2009-06-01 15:40:56Z apdavison $
+$Id: cells.py 700 2010-01-27 15:05:00Z apdavison $
 """
 
 from pyNN import common, cells
@@ -9,6 +9,8 @@ import pypcsim
 import numpy
 import logging
 
+logger = logging.getLogger("PyNN")
+
 class IF_curr_alpha(cells.IF_curr_alpha):
     """Leaky integrate and fire model with fixed threshold and alpha-function-
     shaped post-synaptic current."""
@@ -83,6 +85,7 @@ class IF_cond_alpha(cells.IF_cond_alpha):
     pcsim_name = "LIFCondAlphaNeuron"    
     simObjFactory = None
     setterMethods = {}
+    recordable = ['spikes', 'v']
         
     def __init__(self, parameters):
         cells.IF_cond_alpha.__init__(self, parameters)
@@ -111,6 +114,7 @@ class IF_cond_exp(cells.IF_cond_exp):
     pcsim_name = "LIFCondExpNeuron"    
     simObjFactory = None
     setterMethods = {}
+    recordable = ['spikes', 'v']
         
     def __init__(self, parameters):
         cells.IF_cond_exp.__init__(self, parameters)
@@ -143,12 +147,16 @@ def sanitize_spike_times(spike_times):
     This workaround removes any spikes after the first within a given time step.
     """
     time_step = common.get_time_step()
-    spike_times = numpy.array(spike_times)
+    try:
+        spike_times = numpy.array(spike_times, float)
+    except ValueError, e:
+        raise common.InvalidParameterValueError("Spike times must be floats. %s")
+    
     bins = (spike_times/time_step).astype('int')
     mask = numpy.concatenate((numpy.array([True]), bins[1:] != bins[:-1]))
     if mask.sum() < len(bins):
-        logging.warn("Spikes have been thrown away because they were too close together.")
-        logging.debug(spike_times[(1-mask).astype('bool')])
+        logger.warn("Spikes have been thrown away because they were too close together.")
+        logger.debug(spike_times[(1-mask).astype('bool')])
     if len(spike_times) > 0:
         return spike_times[mask]
     else:
