@@ -22,18 +22,20 @@ Attributes:
 All other functions and classes are private, and should not be used by other
 modules.
     
-$Id: simulator.py 629 2009-06-02 15:50:13Z apdavison $
+$Id: simulator.py 702 2010-02-03 10:39:32Z apdavison $
 """
 
 import logging
 import pypcsim
 import types
 import numpy
-from pyNN import common, recording
+from pyNN import common
 
 recorder_list = []
 connection_managers = []
 
+logger = logging.getLogger("PyNN")
+
 # --- Internal PCSIM functionality -------------------------------------------- 
 
 def is_local(id):
@@ -123,154 +125,6 @@ class ID(long, common.IDMixin):
             else:               
                 setattr(simobj, name, value)
 
-
-# --- For implementation of record_X()/get_X()/print_X() -----------------------
-
-class Recorder(object):
-    """Encapsulates data and functions related to recording model variables."""
-    
-    fieldnames = {'v': 'Vm',
-                  'gsyn': 'psr'}
-    numpy1_0_formats = {'spikes': "%g", # only later versions of numpy support different
-                        'v': "%g",      # formats for different columns
-                        'gsyn': "%g"}
-    formats = {'spikes': 'id t',
-               'v': 'id t v',
-               'gsyn': 'id t ge gi'}
-    
-    def __init__(self, variable, population=None, file=None):
-        """
-        Create a recorder.
-        
-        `variable` -- "spikes", "v" or "gsyn"
-        `population` -- the Population instance which is being recorded by the
-                        recorder (optional)
-        `file` -- one of:
-            - a file-name,
-            - `None` (write to a temporary file)
-            - `False` (write to memory).
-        """
-        self.variable = variable
-        self.filename = file or None
-        self.population = population # needed for writing header information
-        self.recorded = set([])
-        self.recorders = {}
-    
-    def record(self, ids):
-        """Add the cells in `ids` to the set of recorded cells."""
-        logging.debug('Recorder.record(%s)', str(ids))
-        if self.population:
-            ids = set([id for id in ids if id in self.population.local_cells])
-        else:
-            ids = set(ids) # how to decide if the cell is local?
-        new_ids = list( ids.difference(self.recorded) )
-        
-        self.recorded = self.recorded.union(ids)
-        logging.debug('Recorder.recorded = %s' % self.recorded)
-        if self.variable == 'spikes':        
-            for id in new_ids:
-                #if self.population:
-                #    pcsim_id = self.population.pcsim_population[int(id)]
-                #else:
-                pcsim_id = int(id)
-                src_id = pypcsim.SimObject.ID(pcsim_id)
-                rec = net.create(pypcsim.SpikeTimeRecorder(),
-                                 pypcsim.SimEngine.ID(src_id.node, src_id.eng))            
-                net.connect(pcsim_id, rec, pypcsim.Time.sec(0))
-                assert id not in self.recorders
-                self.recorders[id] = rec
-        elif self.variable == 'v':
-            for id in new_ids:
-                #if self.population:
-                #    pcsim_id = self.population.pcsim_population[int(id)]
-                #else:
-                pcsim_id = int(id)
-                src_id = pypcsim.SimObject.ID(pcsim_id)
-                rec = net.create(pypcsim.AnalogRecorder(),
-                                 pypcsim.SimEngine.ID(src_id.node, src_id.eng))
-                net.connect(pcsim_id, Recorder.fieldnames[self.variable], rec, 0, pypcsim.Time.sec(0))
-                self.recorders[id] = rec
-        else:
-            raise NotImplementedError("Recording of %s not implemented." % self.variable)
-
-    def get(self, gather=False, compatible_output=True, offset=None):
-        """Return the recorded data as a Numpy array."""
-        # compatible_output is not used, but is needed for compatibility with the nest2 module.
-        # Does nest2 really need it?
-        if offset is None:
-            if self.population:
-                offset = self.population.first_id
-            else:
-                offset = 0
-                
-        if self.variable == 'spikes':
-            data = numpy.empty((0,2))
-            for id in self.recorded:
-                rec = self.recorders[id]
-                if isinstance(net.object(id), pypcsim.SpikingInputNeuron):
-                    spikes = 1000.0*numpy.array(net.object(id).getSpikeTimes()) # is this special case really necessary?
-                    spikes = spikes[spikes<=state.t]
-                else:
-                    spikes = 1000.0*numpy.array(net.object(rec).getSpikeTimes())
-                spikes = spikes.flatten()
-                spikes = spikes[spikes<=state.t+1e-9]
-                if len(spikes) > 0:    
-                    new_data = numpy.array([numpy.ones(spikes.shape, dtype=int)*(id-offset), spikes]).T
-                    data = numpy.concatenate((data, new_data))           
-        elif self.variable == 'v':
-            data = numpy.empty((0,3))
-            for id in self.recorded:
-                rec = self.recorders[id]
-                v = 1000.0*numpy.array(net.object(rec).getRecordedValues())
-                v = v.flatten()
-                final_v = 1000.0*net.object(id).getVm()
-                v = numpy.append(v, final_v)
-                dt = state.dt
-                t = dt*numpy.arange(v.size)
-                new_data = numpy.array([numpy.ones(v.shape, dtype=int)*(id-offset), t, v]).T
-                data = numpy.concatenate((data, new_data))
-        elif self.variable == 'gsyn':
-            raise NotImplementedError
-        else:
-            raise Exception("Recording of %s not implemented." % self.variable)
-        return data
-
-    def write(self, file=None, gather=False, compatible_output=True):
-        """Write recorded data to file."""
-        data = self.get(gather)
-        filename = file or self.filename
-        recording.rename_existing(filename)
-        try:
-            numpy.savetxt(filename, data, Recorder.numpy1_0_formats[self.variable], delimiter='\t')
-        except AttributeError, errmsg:
-            # we assume the error is due to the lack of savetxt in older versions of numpy and
-            # so provide a cut-down version of that function
-            f = open(filename, 'w')
-            fmt = Recorder.numpy1_0_formats[self.variable]
-            for row in data:
-                f.write('\t'.join([fmt%val for val in row]) + '\n')
-            f.close()
-        if compatible_output:
-            recording.write_compatible_output(filename, filename, self.variable,
-                                              Recorder.formats[self.variable],
-                                              self.population, state.dt)
-
-    def count(self, gather=False):
-        """
-        Return the number of data points for each cell, as a dict. This is mainly
-        useful for spike counts or for variable-time-step integration methods.
-        """
-        N = {}
-        if self.variable == 'spikes':
-            for id in self.recorded:
-                N[id] = net.object(self.recorders[id]).spikeCount()
-        else:
-            raise Exception("Only implemented for spikes.")
-        if gather and state.num_processes > 1:
-            N = recording.gather_dict(N)
-        return N
-              
-
 # --- For implementation of create() and Population.__init__() -----------------
 
 def create_cells(cellclass, cellparams, n, parent=None):
@@ -365,14 +219,18 @@ class ConnectionManager(object):
 
     synapse_target_ids = { 'excitatory': 1, 'inhibitory': 2 }
 
-    def __init__(self, synapse_model=None, parent=None):
+    def __init__(self, synapse_type, synapse_model=None, parent=None):
         """
         Create a new ConnectionManager.
         
+        `synapse_type` -- the 'physiological type' of the synapse, e.g.
+                          'excitatory' or 'inhibitory', or a PCSIM synapse
+                          factory.
         `synapse_model` -- not used. Present for consistency with other simulators.
         `parent` -- the parent `Projection`, if any.
         """
         global connection_managers
+        self.synapse_type = synapse_type
         self.connections = []
         self.parent = parent
         connection_managers.append(self)
@@ -403,7 +261,7 @@ class ConnectionManager(object):
         for i in range(len(self)):
             yield self[i]
     
-    def connect(self, source, targets, weights, delays, synapse_type):
+    def connect(self, source, targets, weights, delays):
         """
         Connect a neuron to one or more other neurons with a static connection.
         
@@ -413,10 +271,6 @@ class ConnectionManager(object):
                      Must have the same length as `targets`.
         `delays`  -- a list/1D array of connection delays, or a single delay.
                      Must have the same length as `targets`.
-        `synapse_type` -- a string identifying the synapse to connect to (should
-                          be "excitatory" or "inhibitory"), or a PCSIM synapse
-                          factory. May be `None`, which is treated the same as
-                          "excitatory".
         """
         if not isinstance(source, (int, long)) or source < 0:
             errmsg = "Invalid source ID: %s" % source
@@ -437,7 +291,7 @@ class ConnectionManager(object):
         else:
             weight_scale_factor = 1e-9 # Convert from nA to A
         
-        synapse_type = synapse_type or "excitatory"
+        synapse_type = self.synapse_type or "excitatory"
         if isinstance(synapse_type, basestring):
             syn_target_id = ConnectionManager.synapse_target_ids[synapse_type]
             syn_factory = pypcsim.SimpleScalingSpikingSynapse(
@@ -493,11 +347,25 @@ class ConnectionManager(object):
         
         `name`  -- attribute name
         `value` -- the attribute numeric value, or a list/1D array of such
-                   values of the same length as the number of local connections.
+                   values of the same length as the number of local connections,
+                   or a 2D array with the same dimensions as the connectivity
+                   matrix (as returned by `get(format='array')`)
         """
         if common.is_number(value):
             for c in self:
                 setattr(c, name, value)
+        elif isinstance(value, numpy.ndarray) and len(value.shape) == 2:
+            offset = (self.parent.pre.first_id, self.parent.post.first_id)
+            for c in self.connections:
+                addr = (c.source-offset[0], c.target-offset[1])
+                try:
+                    val = value[addr]
+                except IndexError, e:
+                    raise IndexError("%s. addr=%s" % (e, addr))
+                if numpy.isnan(val):
+                    raise Exception("Array contains no value for synapse from %d to %d" % (c.source, c.target))
+                else:
+                    setattr(c, name, val)
         elif common.is_listlike(value):
             for c,val in zip(self.connections, value):
                 setattr(c, name, val)
