@@ -9,13 +9,10 @@ Functions:
     init_logging()    - convenience function for setting up logging to file and
                         to the screen.
     
-Classes:
-    MultiSim - a small framework to make it easier to compare results between
-               different simulators.
     Timer    - a convenience wrapper around the time.time() function from the
                standard library.
 
-$Id: utility.py 647 2009-06-09 12:37:02Z apdavison $
+$Id: utility.py 696 2010-01-18 14:25:32Z apdavison $
 """
 
 # If there is a settings.py file on the path, defaults will be
@@ -83,6 +80,7 @@ def get_script_args(n_args, usage=''):
 def init_logging(logfile, debug=False, num_processes=1, rank=0):
     if num_processes > 1:
         logfile += '.%d' % rank
+    logfile = os.path.abspath(logfile)
     if debug:
         logging.basicConfig(level=logging.DEBUG,
                     format='%(asctime)s %(levelname)s %(message)s',
@@ -93,63 +91,7 @@ def init_logging(logfile, debug=False, num_processes=1, rank=0):
                     format='%(asctime)s %(levelname)s %(message)s',
                     filename=logfile,
                     filemode='w')
-        
-        
-class MultiSim(object):
-    """
-    A small framework to make it easier to run the same model on multiple
-    simulators.
-    
-    Currently runs the simulations interleaved, but it would be nice to add
-    parallel runs via threading, multiple processes, or MPI processes.
-    """
-    
-    def __init__(self, sim_list, net, parameters):
-        """
-        Build the model defined in the class `net`, with parameters `parameters`,
-        for each of the simulator modules specified in `sim_list`.
-        
-        The `net` constructor takes arguments `sim` and `parameters`.
-        """
-        self.sim_list = sim_list
-        self.nets = {}
-        for sim in sim_list:
-            self.nets[sim.__name__] = net(sim, parameters)
-            
-    def __iter__(self):
-        return self.nets.itervalues()
-    
-    def __getattr__(self, name):
-        """
-        Assumes `name` is a method of the `net` model.
-        Return a function that runs `net.name()` for all the simulators.
-        """
-        def iterate_over_nets(*args, **kwargs):
-            retvals = {}
-            for sim_name, net in self.nets.items():
-                retvals[sim_name] = getattr(net, name)(*args, **kwargs)
-            return retvals
-        return iterate_over_nets
-            
-    def run(self, simtime, steps=1, *callbacks):
-        """
-        Run the model for a time `simtime` in all simulators.
-        
-        The run may be broken into a number of steps (each of equal duration).
-        Any functions in `callbacks` will be called after each step.
-        """
-        dt = float(simtime)/steps
-        for i in range(steps):
-            for sim in self.sim_list:
-                sim.run(dt)
-            for func in callbacks:
-                func()
-                
-    def end(self):
-        for sim in self.sim_list:
-            sim.end()
-            
-            
+
 def save_population(population,filename,variables=[]):
     """
     Saves the spike_times of a  population and the dim, size, labels such that one can load it back into a SpikeSourceArray population using the load_population function.
@@ -195,18 +137,30 @@ class Timer(object):
     def start(self):
         """Start timing."""
         self._start_time = time.time()
+        self._last_check = self._start_time
     
     def elapsedTime(self, format=None):
         """Return the elapsed time in seconds but keep the clock running."""
-        elapsed_time = time.time() - self._start_time
+        current_time = time.time()
+        elapsed_time = current_time - self._start_time
         if format=='long':
             elapsed_time = Timer.time_in_words(elapsed_time)
+        self._last_check = current_time
         return elapsed_time
     
     def reset(self):
         """Reset the time to zero, and start the clock."""
         self.start()
     
+    def diff(self, format=None): # I think delta() would be a better name for this method.
+        """Return the time since the last time elapsedTime() or diff() was called."""
+        current_time = time.time()
+        time_since_last_check = current_time - self._last_check
+        self._last_check = current_time
+        if format=='long':
+            time_since_last_check = Timer.time_in_words(elapsed_time)
+        return time_since_last_check
+    
     @staticmethod
     def time_in_words(s):
         """Formats a time in seconds as a string containing the time in days,
