@@ -1,25 +1,29 @@
 """
 Unit tests for all simulators
-$Id: generictests.py 637 2009-06-04 14:45:51Z apdavison $
+$Id: generictests.py 702 2010-02-03 10:39:32Z apdavison $
 """
 
 import sys
 import unittest
 import numpy
 import os
-from pyNN import common, random, utility
+import cPickle as pickle
+from pyNN import common, random, utility, recording
+import glob
 
 try:
     from mpi4py import MPI
 except ImportError:
     MPI = None
 
-def assert_arrays_almost_equal(a, b, threshold):
+def assert_arrays_almost_equal(a, b, threshold, msg=''):
     if a.shape != b.shape:
         raise unittest.TestCase.failureException("Shape mismatch: a.shape=%s, b.shape=%s" % (a.shape, b.shape))
     if not (abs(a-b) < threshold).all():
         err_msg = "%s != %s" % (a, b)
         err_msg += "\nlargest difference = %g" % abs(a-b).max()
+        if msg:
+            err_msg += "\nOther information: %s" % msg
         raise unittest.TestCase.failureException(err_msg)
 
 def is_local(cell):
@@ -29,6 +33,9 @@ def is_local(cell):
 class CreationTest(unittest.TestCase):
     """Tests of the create() function."""
     
+    def setUp(self):
+        sim.setup()
+    
     def testCreateStandardCell(self):
         for cellclass in sim.list_standard_models():
             ifcell = sim.create(cellclass)
@@ -69,12 +76,9 @@ class ConnectionTest(unittest.TestCase):
     """Tests of the connect() function."""
     
     def setUp(self):
-        self.precells = sim.create(sim.SpikeSourcePoisson, n=5)
-        self.postcells = sim.create(sim.IF_curr_alpha, n=3)
-        
-        
-    def tearDown(self):
-        pass
+        sim.setup()
+        self.precells = sim.create(sim.SpikeSourcePoisson, n=7)
+        self.postcells = sim.create(sim.IF_curr_alpha, n=5)
         
     def testConnectTwoCells(self):
         conn_list = sim.connect(self.precells[0], self.postcells[0])
@@ -170,9 +174,6 @@ class IDSetGetTest(unittest.TestCase):
             self.cells[cell_class.__name__] = sim.create(cell_class, n=2)
             self.populations[cell_class.__name__] = sim.Population(2, cell_class)
     
-    def tearDown(self):
-        pass
-    
     def testSetGet(self):
         """__setattr__(), __getattr__(): sanity check"""
         for cell_class in IDSetGetTest.model_list:
@@ -283,6 +284,7 @@ class IDSetGetTest(unittest.TestCase):
 class SetValueTest(unittest.TestCase):
     
     def setUp(self):
+        sim.setup()
         self.cells = sim.create(sim.IF_curr_exp, n=10)
         self.single_cell = sim.create(sim.IF_cond_exp, n=1)
         
@@ -316,10 +318,7 @@ class PopulationInitTest(unittest.TestCase):
     """Tests of the __init__() method of the Population class."""
     
     def setUp(self):
-        pass
-    
-    def tearDown(self):
-        pass
+        sim.setup()
         
     def testSimpleInit(self):
         net = sim.Population((3,3), sim.IF_curr_alpha)
@@ -347,6 +346,7 @@ class PopulationIndexTest(unittest.TestCase):
     """Tests of the Population class indexing."""
     
     def setUp(self):
+        sim.setup()
         self.net1 = sim.Population((10,), sim.IF_curr_alpha)
         self.net2 = sim.Population((2,4,3), sim.IF_curr_exp)
         self.net3 = sim.Population((2,2,1), sim.SpikeSourceArray)
@@ -383,6 +383,7 @@ class PopulationIteratorTest(unittest.TestCase):
     """Tests of the Population class iterators."""
     
     def setUp(self):
+        sim.setup()
         self.net1 = sim.Population((10,), sim.IF_curr_alpha)
         self.net2 = sim.Population((2,4,3), sim.IF_curr_exp)
         self.net3 = sim.Population((2,2,1), sim.SpikeSourceArray)
@@ -408,9 +409,10 @@ class PopulationSpikesTest(unittest.TestCase):
     def setUp(self):
         sim.setup()
         self.spiketimes = numpy.arange(5,105,10.0)
-        spiketimes_2D = self.spiketimes.reshape((len(self.spiketimes),1))
-        self.input_spike_array = numpy.concatenate((numpy.zeros(spiketimes_2D.shape, 'float'), spiketimes_2D),
-                                                   axis=1)
+        #spiketimes_2D = self.spiketimes.reshape((len(self.spiketimes),1))
+        #self.input_spike_array = numpy.concatenate((numpy.zeros(spiketimes_2D.shape, 'float'), spiketimes_2D),
+        #                                           axis=1)
+        self.input_spike_array = self.spiketimes
         self.p1 = sim.Population(1, sim.SpikeSourceArray, {'spike_times': self.spiketimes})
     
     def tearDown(self):
@@ -419,7 +421,7 @@ class PopulationSpikesTest(unittest.TestCase):
     def testGetSpikes(self):
         self.p1.record()
         sim.run(100.0)
-        output_spike_array = self.p1.getSpikes()
+        output_spike_array = self.p1.getSpikes()[:,1]
         if sim.rank() == 0:
             assert_arrays_almost_equal(self.input_spike_array, output_spike_array, 1e-11)
     
@@ -431,9 +433,9 @@ class PopulationSpikesTest(unittest.TestCase):
         self.p1.record()
         sim.run(100.0)
         output_spike_array = self.p1.getSpikes()
-        self.assertEqual(self.input_spike_array.shape, (10,2))
+        self.assertEqual(self.input_spike_array.shape, (10,))
         if sim.rank() == 0:
-            self.assertEqual(self.input_spike_array.shape, output_spike_array.shape)
+            self.assertEqual(self.input_spike_array.size, output_spike_array.shape[0])
 
 #===============================================================================
 class PopulationSetTest(unittest.TestCase):
@@ -515,28 +517,31 @@ class PopulationSetTest(unittest.TestCase):
         array_in = numpy.array([[0.1,0.2,0.3],[0.4,0.5,0.6]])
         self.assertRaises(common.InvalidDimensionsError, self.p1.tset, 'i_offset', array_in)
     
-    #def testTSetInvalidValues(self):
-    #    """Population.tset(): If some of the values in the valueArray are invalid, should raise an exception."""
-    #    array_in = numpy.array([['potatoes','carrots'],['oranges','bananas']])
-    #    self.assertRaises(common.InvalidParameterValueError, self.p2.tset, 'spike_times', array_in)
+    def testTSetInvalidValues(self):
+        """Population.tset(): If some of the values in the valueArray are invalid, should raise an exception."""
+        array_in = numpy.array([['potatoes','carrots'],['oranges','bananas']])
+        self.assertRaises(common.InvalidParameterValueError, self.p2.tset, 'spike_times', array_in)
         
     def testRSetNumpy(self):
         """Population.rset(): with numpy rng."""
-        rd1 = random.RandomDistribution(rng=random.NumpyRNG(seed=98765),
+        rd1 = random.RandomDistribution(rng=random.NumpyRNG(seed=98765, num_processes=sim.num_processes()),
                                          distribution='uniform',
                                          parameters=[0.9,1.1])
-        rd2 = random.RandomDistribution(rng=random.NumpyRNG(seed=98765),
+        rd2 = random.RandomDistribution(rng=random.NumpyRNG(seed=98765, num_processes=sim.num_processes()),
                                          distribution='uniform',
                                          parameters=[0.9,1.1])
         self.p1.rset('cm', rd1)
         output_values = self.p1.get('cm', as_array=True)
         mask = (1-numpy.isnan(output_values)).astype(bool)
-        input_values = rd2.next(len(self.p1)).reshape(self.p1.dim)
+        input_values = rd2.next(len(self.p1), mask_local=False).reshape(self.p1.dim)
         assert_arrays_almost_equal(input_values[mask], output_values[mask], 1e-7)
                 
 #===============================================================================                
 class PopulationPositionsTest(unittest.TestCase):
     
+    def setUp(self):
+        sim.setup()
+    
     def test_nearest(self):
         p = sim.Population((4,5,6), sim.IF_cond_exp)
         self.assertEqual(p.nearest((0.0,0.0,0.0)), p[0,0,0])
@@ -551,6 +556,9 @@ class PopulationPositionsTest(unittest.TestCase):
 #===============================================================================
 class PopulationCellAccessTest(unittest.TestCase):
     
+    def setUp(self):
+        sim.setup()
+        
     def test_index(self):
         p = sim.Population((4,5,6), sim.IF_cond_exp)
         self.assertEqual(p.index(0), p[0,0,0])
@@ -563,17 +571,15 @@ class PopulationRecordTest(unittest.TestCase): # to write later
        meanSpikeCount() methods of the Population class."""
     
     def setUp(self):
+        sim.setup()
         self.pop1 = sim.Population((3,3), sim.SpikeSourcePoisson,{'rate': 20})
         self.pop2 = sim.Population((3,3), sim.IF_curr_alpha)
         self.pop3 = sim.Population((3,3), sim.IF_cond_alpha)
         #self.pop4 = sim.Population((3,3), sim.EIF_cond_alpha_isfa_ista)
-        self.pops =[self.pop1, self.pop2, self.pop3] #, self.pop3]
-        if hasattr(sim, 'simulator') and hasattr(sim.simulator, 'reset'):
-            sim.simulator.reset()
+        self.pops = [self.pop1, self.pop2, self.pop3]
 
-    #def tearDown(self):
-    #    if hasattr(sim, 'simulator') and hasattr(sim.simulator, 'reset'):
-    #        sim.simulator.reset()
+    def tearDown(self):
+        sim.end()
 
     def testRecordAll(self):
         """Population.record(): not a full test, just checking there are no Exceptions raised."""
@@ -610,8 +616,9 @@ class PopulationRecordTest(unittest.TestCase): # to write later
         #self.pop3.record()
         simtime = 1000.0
         sim.run(simtime)
-        rate = self.pop1.meanSpikeCount()*1000/simtime
+        msc = self.pop1.meanSpikeCount()
         if sim.rank() == 0: # only on master node
+            rate = msc*1000/simtime
             ##print self.pop1.recorders['spikes'].recorders
             assert (20*0.8 < rate) and (rate < 20*1.2), "rate is %s" % rate
         #rate = self.pop3.meanSpikeCount()*1000/simtime
@@ -628,7 +635,6 @@ class PopulationRecordTest(unittest.TestCase): # to write later
         cells_to_record = [self.pop2[0,0], self.pop2[1,1]]
         self.pop2.record_v(cells_to_record)
         simtime = 10.0
-        sim.running = False
         sim.run(simtime)
         self.pop2.print_v("temp.v", gather=True, compatible_output=True)
         vm = self.pop2.get_v()
@@ -637,6 +643,7 @@ class PopulationRecordTest(unittest.TestCase): # to write later
             vm_fromfile = numpy.loadtxt("temp.v")
             #print vm_fromfile
             self.assertEqual(vm_fromfile.shape, ((1+int(10.0/0.1))*len(cells_to_record), 2))
+            os.remove("temp.v")
         
     def testSynapticConductanceRecording(self):
         # current-based synapses
@@ -645,7 +652,6 @@ class PopulationRecordTest(unittest.TestCase): # to write later
         cells_to_record = [self.pop3[1,0], self.pop3[2,2]]
         self.pop3.record_gsyn(cells_to_record)
         simtime = 10.0
-        sim.running = False
         sim.run(simtime)
         gsyn = self.pop3.get_gsyn()
         if sim.rank() == 0:
@@ -660,7 +666,7 @@ class PopulationRecordTest(unittest.TestCase): # to write later
         spike_times = numpy.arange(10.0, 200.0, 10.0)
         spike_source = sim.Population(1, sim.SpikeSourceArray, {'spike_times': spike_times})
         spike_source.record()
-        sim.run(100.0)
+        sim.run(101.0)
         spikes = spike_source.getSpikes()
         spikes = spikes[:,1]
         if sim.rank() == 0:
@@ -709,6 +715,9 @@ class ProjectionTest(unittest.TestCase):
         self.prj.printWeights("weights_list.tmp", format='list', gather=False)
         self.prj.printWeights("weights_array.tmp", format='array', gather=False)
         # test needs completing. Should read in the weights and check they have the correct values
+        for filename in "weights_list.tmp", "weights_array.tmp":
+            if os.path.exists(filename):
+                os.remove(filename)
          
     def test_iterator(self):
         assert hasattr(self.prj, "connections")
@@ -846,6 +855,9 @@ class ProjectionInitTest(unittest.TestCase):
             d2.append(c2.delay)
         assert (w1 == w2), 'w1 = %s\nw2 = %s' % (w1, w2)
         assert (d1 == d2), 'd1 = %s\nd2 = %s' % (d1, d2)
+        if sim.rank() == 0:
+            for filename in glob.glob("connections.tmp*"):
+                os.remove(filename)
           
     def testSettingDelays(self):
         """Delays should be set correctly when using a Connector object."""
@@ -863,56 +875,92 @@ class ProjectionSetTest(unittest.TestCase):
     randomizeDelays() methods of the Projection class."""
 
     def setUp(self):
-        self.target   = sim.Population((3,3), sim.IF_curr_alpha)
+        sim.setup()
+        self.target_curr = sim.Population((3,3), sim.IF_curr_alpha)
+        self.target_cond = sim.Population(5, sim.IF_cond_exp)
+        self.targets = (self.target_curr, self.target_cond)
         self.source   = sim.Population((3,3), sim.SpikeSourcePoisson,{'rate': 200})
         self.distrib_Numpy = random.RandomDistribution(rng=random.NumpyRNG(12345), distribution='uniform', parameters=(0.2,1)) 
         self.distrib_Native= random.RandomDistribution(rng=random.NativeRNG(12345), distribution='uniform', parameters=(0.2,1)) 
         
-    def testSetWeights(self):
-        prj1 = sim.Projection(self.source, self.target, sim.AllToAllConnector())
-        prj1.setWeights(2.345)
+    def testSetPositiveWeights(self):
+        prj1 = sim.Projection(self.source, self.target_curr, sim.AllToAllConnector(), target='excitatory', label="exc, curr")
+        prj2 = sim.Projection(self.source, self.target_curr, sim.AllToAllConnector(), target='inhibitory', label="inh, curr")
+        prj3 = sim.Projection(self.source, self.target_cond, sim.AllToAllConnector(), target='excitatory', label="exc, cond")
+        prj4 = sim.Projection(self.source, self.target_cond, sim.AllToAllConnector(), target='inhibitory', label="inh, cond")
+        for prj in prj1, prj3, prj4:
+            prj.setWeights(2.345)
+            weights = []
+            for c in prj.connections:
+                weights.append(c.weight)
+            result = 2.345*numpy.ones(len(prj.connections))
+            assert_arrays_almost_equal(numpy.array(weights), result, 1e-7, msg=prj.label)
+        self.assertRaises(common.InvalidWeightError, prj2.setWeights, 2.345) # current-based inhibitory needs negative weights
+            
+    def testSetNegativeWeights(self):
+        prj1 = sim.Projection(self.source, self.target_curr, sim.AllToAllConnector(), target='excitatory')
+        prj2 = sim.Projection(self.source, self.target_curr, sim.AllToAllConnector(), target='inhibitory')
+        prj3 = sim.Projection(self.source, self.target_cond, sim.AllToAllConnector(), target='excitatory')
+        prj4 = sim.Projection(self.source, self.target_cond, sim.AllToAllConnector(), target='inhibitory')
+        prj2.setWeights(-2.345)
         weights = []
-        for c in prj1.connections:
+        for c in prj2.connections:
             weights.append(c.weight)
-        result = 2.345*numpy.ones(len(prj1.connections))
+        result = -2.345*numpy.ones(len(prj2.connections))
         assert_arrays_almost_equal(numpy.array(weights), result, 1e-7)
-        
+        for prj in prj1, prj3, prj4:
+            self.assertRaises(common.InvalidWeightError, prj.setWeights, -2.345) 
+    
+    def test_set_weights_with_array(self):
+        prj = sim.Projection(self.source, self.target_curr,
+                             sim.FixedProbabilityConnector(0.5, weights=self.distrib_Numpy),
+                             target='excitatory')
+        weight_array = prj.getWeights(format='array')
+        prj.setWeights(weight_array + 0.5)
+        def filter_NaN(arr):
+            nan_filter = (1-numpy.isnan(arr)).astype(bool)
+            return arr[nan_filter]
+        assert_arrays_almost_equal(filter_NaN(weight_array + 0.5), filter_NaN(prj.getWeights(format='array')), 1e-7)
+    
     def testSetDelays(self):
-        prj1 = sim.Projection(self.source, self.target, sim.AllToAllConnector())
-        prj1.setDelays(2.345)
-        delays = []
-        for c in prj1.connections:
-            delays.append(c.delay)
-        if sim_name != 'nest':
-            result = 2.345*numpy.ones(len(prj1.connections))
-        else:
-            result = 2.4*numpy.ones(len(prj1.connections)) # nest rounds delays up
-        assert_arrays_almost_equal(numpy.array(delays), result, 1e-7)
-        
+        for target in self.targets:
+            prj1 = sim.Projection(self.source, target, sim.AllToAllConnector())
+            prj1.setDelays(2.345)
+            delays = []
+            for c in prj1.connections:
+                delays.append(c.delay)
+            if sim_name != 'nest':
+                result = 2.345*numpy.ones(len(prj1.connections))
+            else:
+                result = 2.4*numpy.ones(len(prj1.connections)) # nest rounds delays up
+            assert_arrays_almost_equal(numpy.array(delays), result, 1e-7)
+            
     def testRandomizeWeights(self):
         # The probability of having two consecutive weight vectors that are equal should be effectively 0
-        prj1 = sim.Projection(self.source, self.target, sim.AllToAllConnector())
-        prj1.randomizeWeights(self.distrib_Numpy)
-        w1 = []; w2 = [];
-        for c in prj1.connections:
-            w1.append(c.weight)
-        prj1.randomizeWeights(self.distrib_Numpy)        
-        for c in prj1.connections:
-            w2.append(c.weight)
-        self.assertNotEqual(w1,w2)
-        self.assertEqual(w2[0], prj1.connections[0].weight)
-        
+        for target in self.targets:
+            prj1 = sim.Projection(self.source, target, sim.AllToAllConnector())
+            prj1.randomizeWeights(self.distrib_Numpy)
+            w1 = []; w2 = [];
+            for c in prj1.connections:
+                w1.append(c.weight)
+            prj1.randomizeWeights(self.distrib_Numpy)        
+            for c in prj1.connections:
+                w2.append(c.weight)
+            self.assertNotEqual(w1,w2)
+            self.assertEqual(w2[0], prj1.connections[0].weight)
+            
     def testRandomizeDelays(self):
         # The probability of having two consecutive delay vectors that are equal should be effectively 0
-        prj1 = sim.Projection(self.source, self.target, sim.FixedProbabilityConnector(0.8))
-        prj1.randomizeDelays(self.distrib_Numpy)
-        d1 = []; d2 = [];
-        for c in prj1.connections:
-            d1.append(c.delay)
-        prj1.randomizeDelays(self.distrib_Numpy)        
-        for c in prj1.connections:
-            d2.append(c.delay)
-        self.assertNotEqual(d1,d2)
+        for target in self.targets:
+            prj1 = sim.Projection(self.source, target, sim.FixedProbabilityConnector(0.8))
+            prj1.randomizeDelays(self.distrib_Numpy)
+            d1 = []; d2 = [];
+            for c in prj1.connections:
+                d1.append(c.delay)
+            prj1.randomizeDelays(self.distrib_Numpy)        
+            for c in prj1.connections:
+                d2.append(c.delay)
+            self.assertNotEqual(d1,d2)
          
 #===============================================================================
 class ProjectionGetTest(unittest.TestCase):
@@ -966,6 +1014,9 @@ class ConnectorsTest(unittest.TestCase):
 #===============================================================================
 class ElectrodesTest(unittest.TestCase):
     
+    def setUp(self):
+        sim.setup()
+        
     def test_DCSource(self):
         # just check no Exceptions are raised, for now.
         source = sim.DCSource(amplitude=0.5, start=50.0, stop=100.0)
@@ -988,16 +1039,15 @@ class ElectrodesTest(unittest.TestCase):
 class StateTest(unittest.TestCase):
     
     def setUp(self):
+        sim.setup()
         self.cells = sim.create(sim.IF_curr_exp) # brian chokes if there are no NeuronGroups in the Network object when net.run() is called
     
     def test_get_time(self):
-        sim.setup()
         self.assertEqual(sim.get_current_time(), 0.0)
         sim.run(100.0)
         self.assertAlmostEqual(sim.get_current_time(), 100.0, 9)
         
     def test_get_time_step(self):
-        sim.setup()
         self.assertEqual(sim.get_time_step(), 0.1)
         sim.setup(timestep=0.05)
         self.assertEqual(sim.get_time_step(), 0.05)
@@ -1006,6 +1056,7 @@ class StateTest(unittest.TestCase):
 class SpikeSourceTest(unittest.TestCase):
     
     def test_end_of_poisson(self):
+        sim.setup()
         # see ticket:127
         start = 100.0
         rate = 50.0
@@ -1016,20 +1067,86 @@ class SpikeSourceTest(unittest.TestCase):
         PNs.record()
         sim.run(simtime)
         spikes = PNs.getSpikes()[:,1]
-        assert min(spikes) >= start, min(spikes)
-        assert max(spikes) <= start+duration, "%g > %g" % (max(spikes), start+duration)
-        expected_count = duration/1000.0*rate*n
-        diff = abs(len(spikes)-expected_count)/expected_count
-        assert diff < 0.1, str(diff)
+        if sim.rank() == 0:
+            assert min(spikes) >= start, min(spikes)
+            assert max(spikes) <= start+duration, "%g > %g" % (max(spikes), start+duration)
+            expected_count = duration/1000.0*rate*n
+            diff = abs(len(spikes)-expected_count)/expected_count
+            assert diff <= 0.1, "diff = %g. Expected count = %d, actual count = %d" % (diff, expected_count, len(spikes))
     
+class FileTest(unittest.TestCase):
+    
+    def setUp(self):
+        sim.setup()
+        self.pop = sim.Population((3,3), sim.IF_curr_alpha)
+        rng = random.NumpyRNG(123)
+        v_reset  = -65.0
+        v_thresh = -50.0
+        uniformDistr = random.RandomDistribution(rng=rng, distribution='uniform', parameters=[v_reset, v_thresh])
+        self.pop.randomInit(uniformDistr)
+        self.cells_to_record = [self.pop[0,0], self.pop[1,1]]
+        self.pop.record_v(self.cells_to_record)
+        simtime = 10.0
+        sim.run(simtime)
+        
+    def tearDown(self):
+        if sim.rank() == 0:
+            for ext in "txt", "pkl", "npz", "h5":
+                filename = "temp_v.%s" % ext
+                if os.path.exists(filename):
+                    os.remove(filename)
+    
+    def test_text_file(self):
+        output_file = recording.files.StandardTextFile("temp_v.txt", 'w')
+        self.pop.print_v(output_file, gather=True, compatible_output=True)
+        vm = self.pop.get_v()
+        if sim.rank() == 0:
+            self.assertEqual(vm.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 3))
+            vm_fromfile = numpy.loadtxt(output_file.name)
+            #print vm_fromfile
+            self.assertEqual(vm_fromfile.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 2))
+                
+    def test_pickle_file(self):
+        output_file = recording.files.PickleFile("temp_v.pkl", 'w')
+        self.pop.print_v(output_file, gather=True, compatible_output=True)
+        vm = self.pop.get_v()
+        if sim.rank() == 0:
+            self.assertEqual(vm.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 3))
+            f = open(output_file.name)
+            vm_fromfile, metadata = pickle.load(f)
+            f.close()
+            #print vm_fromfile
+            self.assertEqual(vm_fromfile.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 2))
+        
+    def test_numpy_binary_file(self):
+        output_file = recording.files.NumpyBinaryFile("temp_v.npz", 'w')
+        self.pop.print_v(output_file, gather=True, compatible_output=True)
+        vm = self.pop.get_v()
+        if sim.rank() == 0:
+            self.assertEqual(vm.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 3))
+            vm_fromfile = numpy.load(output_file.name)['data']
+            #print vm_fromfile
+            self.assertEqual(vm_fromfile.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 2))
+                
+    def test_hdf5_array_file(self):
+        output_file = recording.files.HDF5ArrayFile("temp_v.h5", 'w')
+        self.pop.print_v(output_file, gather=True, compatible_output=True)
+        vm = self.pop.get_v()
+        if sim.rank() == 0:
+            self.assertEqual(vm.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 3))
+            #h5file = tables.openFile(output_file.name, 'r')
+            h5file = recording.files.HDF5ArrayFile("temp_v.h5", 'r')
+            vm_fromfile = h5file.read()
+            h5file.close()
+            self.assertEqual(vm_fromfile.shape, ((1+int(10.0/0.1))*len(self.cells_to_record), 2))
                 
 # ==============================================================================
 if __name__ == "__main__":
-    sim_name = utility.get_script_args(__file__, 1)[0]
+    sim_name = utility.get_script_args(1)[0]
     
     sys.argv.remove(sim_name) # because unittest.main() processes sys.argv
     if sim_name == 'neuron':
-        sys.argv = sys.argv[sys.argv.index(__file__):]
+        sys.argv = sys.argv[sys.argv.index('generictests.py'):]
     
     #print sys.argv
     sim = __import__("pyNN.%s" % sim_name, None, None, [sim_name])
